#!/usr/bin/env python

"""
Usage: 

merge_lines_periodic.py i1 i2 i3... \
          [-p natoms_per_monomer] \
          [-s nskip] [-d delim_atom] [-D delim_monomer] \
          < crds_input.raw  \
          > multiple_atom_crds_per_line.dat

Explanation: This script splits a text file into equally sized "blocks" 
             (aka "monomers") and pastes the text text from different 
             lines in each block into the same line (with optional delimeters).
             The i1 i2 i3,... indices select the lines in each block (of atom
             coordinates in each monomer) that you want to merge together.
             Indexing begins at 0, not 1.  (The first line in a block has i=0)
         -Negative numbers correspond to atoms in the previous block(monomer).
         -Numbers larger than natoms_per_monomer lie in the next block(monomer).
             If any of these indices lie out of range, then the entire list 
             of lines in this block is ignored.
         -The -p argument indicates the number of lines in each block (aka "monomer")
             If the -p argument is skipped, then it is assumed to be infinity. (In other
             words, it is equal to the number of lines in the polymer conformation.)
         -The -s nskip argument allows you to skip over lines at the begining
             of the file.  (NOTE: Comments and lines beginning with comments 
             are ignored already, so don't include them in the nskip argument.)
         -The -d and -D delimeters allow you to change the string which 
             separates text belonging to different atoms(lines), and different 
             monomers (blocks). By default, they are " " and "\\n", respectively.
         -Blank lines (if present) in the input file are interpreted as delimeters 
             separating different "polymer conformations".  When encountered, each 
             "polymer conformation" is processed separately, with the output for 
             different polymer conformations delimted by blank lines.
"""


import sys

g_filename    = __file__.split('/')[-1]
g_module_name  = g_filename
g_program_name = g_filename
if g_filename.rfind('.py') != -1:
    g_module_name = g_filename[:g_filename.rfind('.py')]
g_date_str     = '2017-2-12'
g_version_str  = '0.2.1'




class InputError(Exception):
    """ A generic exception object containing a string for error reporting.
        (Raising this exception implies that the caller has provided
         a faulty input file or argument.)

    """

    def __init__(self, err_msg):
        self.err_msg = err_msg
    def __str__(self):
        return self.err_msg
    def __repr__(self):
        return str(self)


def EscCharStrToChar(s_in, escape='\\'):
    """ 
    EscCharStrToChar() replaces any escape sequences 
    in a string with their 1-character equivalents.

    """
    assert(len(escape) > 0)
    out_lstr = []
    escaped_state = False
    for c in s_in:
        if escaped_state:
            if (c == 'n'):
                out_lstr.append('\n')
            elif (c == 't'):
                out_lstr.append('\t')
            elif (c == 'r'):
                out_lstr.append('\r')
            elif (c == 'f'):
                out_lstr.append('\f')
            elif (c == '\''):
                out_lstr.append('\'')
            elif (c == '\"'):
                out_lstr.append('\"')
            elif c in escape:
                out_lstr.append(c)
            else:
                out_lstr.append(escape+c) # <- keep both characters
            escaped_state = False
        else:
            if c in escape:
                escaped_state = True
            else:
                out_lstr.append(c)

    return ''.join(out_lstr)



def SafelyEncodeString(in_str, 
                       quotes='\'\"',
                       delimiters=' \t\r\f\n', 
                       escape='\\', 
                       comment_char='#'):
    """
    SafelyEncodeString(in_str) scans through the input string (in_str),
    and returns a new string in which problematic characters 
    (like newlines, tabs, quotes, etc), are replaced by their two-character
    backslashed equivalents (like '\n', '\t', '\'', '\"', etc).  
    The escape character is the backslash by default, but it too can be
    overridden to create custom escape sequences 
    (but this does not effect the encoding for characters like '\n', '\t').

    """
    assert(len(escape) > 0)
    out_lstr = []
    use_outer_quotes = False
    for c in in_str:
        if (c == '\n'):
            c = '\\n'
        elif (c == '\t'):
            c = '\\t'
        elif (c == '\r'):
            c = '\\r'
        elif (c == '\f'):
            c = '\\f'
        elif c in quotes:
            c = escape[0]+c
        elif c in escape:
            c = c+c
        elif c in delimiters:
            use_outer_quotes = True
        # hmm... that's all that comes to mind.  Did I leave anything out?
        out_lstr.append(c)

    if use_outer_quotes:
        out_lstr = ['\"'] + out_lstr + ['\"']

    return ''.join(out_lstr)



def ProcessSnapshot(lines, 
                    out_file, 
                    offsets, 
                    period, 
                    nskip, 
                    delimeter_atom,
                    delimeter_monomer):

    offsets_min = min(offsets)
    offsets_max = max(offsets)
    if period == 0:
        num_monomers = 1
    else:
        num_monomers = (len(lines)-nskip)/period
    for I in range(0, num_monomers):

        # If any of the entries will be missing, then ignore the whole list
        # of atoms (lines) for this monomer (block).
        if (I*period + offsets_min < nskip):
            continue
        if (I*period + offsets_max >= len(lines)):
            continue

        for J in range(0, len(offsets)):
            j = offsets[J]
            i = (I*period + nskip) + j
            if (nskip <= i) and (i < len(lines)):
                out_file.write(lines[i])
                if J+1 < len(offsets):
                    out_file.write(delimeter_atom)
                else:
                    out_file.write(delimeter_monomer)



def main():

    g_period = 0
    g_nskip = 0
    g_delimeter_atom = ' '
    g_delimeter_monomer = '\n'
    g_delimeter_snapshot = '\n'
    g_offsets = []

    #######  Main Code Below: #######

    sys.stderr.write(g_program_name+' v'+g_version_str+' '+g_date_str+' ')
    if sys.version < '3':
        sys.stderr.write(' (python version < 3)\n')
    else:
        sys.stderr.write('\n')

    try:

        argv = [arg for arg in sys.argv]

        # Loop over the remaining arguments not processed yet.
        # These arguments are specific to the lttree.py program
        # and are not understood by ttree.py:
        i = 1
        while i < len(argv):
            #sys.stderr.write('argv['+str(i)+'] = \"'+argv[i]+'\"\n')
            if ((argv[i].lower() == '-?') or
                (argv[i].lower() == '--?') or
                (argv[i].lower() == '-help') or
                (argv[i].lower() == '-help')):
                if i+1 >= len(argv):
                    sys.stdout.write("\n Usage:\n\n"+__doc__+'\n')
                    sys.exit(0)

            elif argv[i].lower() == '-p':
                if i+1 >= len(argv):
                    raise InputError('Error: '+argv[i]+' flag should be followed by a number.\n')
                g_period = int(argv[i+1])
                sys.stderr.write('   period = '+str(g_period)+'\n')
                del(argv[i:i+2])

            elif argv[i].lower() == '-s':
                if i+1 >= len(argv):
                    raise InputError('Error: '+argv[i]+' flag should be followed by a number.\n')
                g_nskip = float(argv[i+1])
                sys.stderr.write('   skip first '+str(g_nskip)+' non-comment lines\n')
                del(argv[i:i+2])

            elif argv[i].lower() == '-d':
                if i+1 >= len(argv):
                    raise InputError('Error: '+argv[i]+' flag should be followed by a string.\n')
                g_delimeter_atom = EscCharStrToChar(argv[i+1])
                sys.stderr.write('   delimeter_atom = \"'+SafelyEncodeString(g_delimeter_atom)+'\"\n')
                del(argv[i:i+2])

            elif argv[i].lower() == '-D':
                if i+1 >= len(argv):
                    raise InputError('Error: '+argv[i]+' flag should be followed by string.\n')
                g_delimeter_atom = EscCharStrToChar(argv[i+1])
                sys.stderr.write('   delimeter_monomer = \"'+SafelyEncodeString(g_delimeter_monomer)+'\"\n')
                del(argv[i:i+2])

            elif argv[i][0] == '-':
                # Note: It could be a negative integer, so check for 
                # that before printing an error message
                if not argv[i][1:].isdigit():
                    raise InputError('Error('+g_program_name+'):\n'
                                     'Unrecogized command line argument \"'+argv[i]+'\"\n')
                i += 1

            else:
                i += 1

        if len(argv) == 1:
            raise InputError("Error: Expected a list of integers.\n\n"+
                             "Usage: \n\n"+
                             "       "+__doc__+"\n")

        g_offsets = [int(arg) for arg in argv[1:]]





        # --- Now (finally) read the lines in the standard input ----
        n_snapshots = 0
        lines = []
        in_file = sys.stdin
        for line_orig in in_file:

            ic = line_orig.find('#')
            if ic != -1:
                line = line_orig[:ic]
            else:
                line = line_orig.rstrip('\n')    

            # Blank lines in a trajectory file usually signal the end of the
            # coordinates for that snapshot in the trajectory, and the beginning
            # of the next snapshot.

            if len(line_orig.strip()) == 0:
                if n_snapshots > 0:
                    sys.stdout.write(g_delimeter_snapshot)
                if len(lines) > 0:
                    ProcessSnapshot(lines, 
                                    sys.stdout,
                                    g_offsets, 
                                    g_period,
                                    g_nskip, 
                                    g_delimeter_atom,
                                    g_delimeter_monomer)
                    n_snapshots += 1
                # Clear the lines buffer to begin reading the new snapshot
                del lines[:]
            else:
                if len(line.strip()) > 0:
                    lines.append(line)

        if len(lines) > 0:
            if n_snapshots > 0:
                sys.stdout.write(g_delimeter_snapshot)
            # After reading all of the lines in the file, deal with any lines 
            # left over since reading the last frame
            ProcessSnapshot(lines, 
                            sys.stdout,
                            g_offsets, 
                            g_period,
                            g_nskip, 
                            g_delimeter_atom,
                            g_delimeter_monomer)


    except (ValueError, InputError) as err:
        sys.stderr.write('\n\n'+str(err)+'\n')
        sys.exit(-1)





if __name__ == "__main__":
    main()



#!/usr/bin/env python

# This program finds the location of each TURN in a PDB file.
# (For each turn, it prints out the starting and ending residue
#  using a format which is recognized by the "select_interval.py" script.
#  That script is typically used for extracting excerpts of a PDB file.)
# This script is not really intended to be used on it's own.

import sys

def main():
    for line in sys.stdin:
        if (line[0:6] == "TURN  "):
            initChainID = line[19:20]
            initSeqNum  = int(line[20:24])
            initICode   = line[24:25]
            endChainID  = line[30:31]
            endSeqNum   = int(line[31:35])
            endICode    = line[35:36]
            sys.stdout.write("\""+initChainID+"\" "+str(initSeqNum)+" \""+initICode+
                             "\"  \""+
                             endChainID+"\" "+str(endSeqNum)+" \""+endICode+"\"\n")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This script throws away the leading and trailing characters on each line.
It truncates the first and last a,b characters from each line, where a,b 
are arguments from sys.argv.  (In retrospect, I should have used awk.)
"""

import sys

def main():
    if (len(sys.argv) <= 1):
        sys.stderr.write('Error: expected the number of truncations as an argument\n')
    truncate_a = int(sys.argv[1])
    if (len(sys.argv) >= 3):
        truncate_b = int(sys.argv[2])
    else:
        truncate_b = truncate_a

    for line in sys.stdin:
        line = line.strip()
        if (len(line) > (truncate_a + truncate_b)):
            truncated_line = line[truncate_a:len(line)-truncate_b]
        else:
            truncated_line = ''
        sys.stdout.write(truncated_line+'\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program extracts the information from a PDB file
which lies within an interval in the PDB file.

Usage:

select_interval.py select_interval.py A 3 " " A 10 z < 309d.pdb > 309d_A3-10.pdb

(This selects residues 3-10 from chain A of the PDB file)

Intervals are pairs of amino acids signifying the first and last
residue in the interval.
In order to specify any amino acid in a PDB file, you must provide 3
identifiers:
  the ChainID a single letter specifying (typically " " or A-Z)
  the SeqNum an integer indicating the location within that chain
  the ICode (a single character code for residue insertions.
             Usually this charcter is the space character " ".
             The smallest possible value is " ".  The largest is "z".
             For the record, I hate this file format.)

All 3 identifiers are needed for both the starting and ending residues.
Consequently this program expects 6 arguments: 
   (to be read from the command line)
ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last
"""

import sys
# Sometimes this program pipes its output to other programs which stops reading
# the PDB file prematurely (such as when multiple MODEL records are present).
# Below we silently suppress the ugly "Broken pipe" message this generates:
import signal
signal.signal(signal.SIGPIPE, signal.SIG_DFL)
from resid import *


def main():
    if len(sys.argv) != 7:
        sys.stderr.write("Error: This program requires 6 arguments.\n"
                         "       This program requires a pair of residues to designate the first and\n"
                         "       last members of the interval.  Each residue requires 3 identifiers.\n"
                         "       Consequently the six arguments needed are:\n"
                         "ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last\n")
        exit(-1)

    print('\"'+sys.argv[1] + '\" \"' + sys.argv[2] + '\" \"'+ sys.argv[3]+'\"')
    first = ResID(sys.argv[1], int(sys.argv[2]), sys.argv[3])
    last  = ResID(sys.argv[4], int(sys.argv[5]), sys.argv[6])

    for line in sys.stdin:
        line_type = line[0:6]

        if line_type in set(["ATOM  ", "HETATM", "ANISOU", "SIGATM", "SIGUIJ"]):
            #atomID    = int(line[6:11])
            #atomType  = line[12:16]
            #altLoc    = line[16:17]
            #resType   = line[17:20]
            chainID   = line[21:22]
            seqNum    = line[22:26]
            iCode     = line[26:27]
            resID = ResID(chainID, int(seqNum), iCode)
            if (first <= resID <= last):
                sys.stdout.write(line)

        elif (line_type == "HET   "):
            #hetID     = line[7:10]
            chainID   = line[12:13]
            seqNum    = line[13:17]
            iCode     = line[17:18]
            #numHETATMs = int(line[20:25])
            #descriptor = line[30:70]
            resID = ResID(chainID, int(seqNum), iCode)
            if (first <= resID <= last):
                sys.stdout.write(line)

        elif (line_type == "HELIX "):
            initChainID = line[19:20]
            initSeqNum  = int(line[21:25])
            initICode   = line[25:26]
            initID = ResID(initChainID, int(initSeqNum), initICode)
            endChainID  = line[31:32]
            endSeqNum   = int(line[33:37])
            endICode    = line[37:38]
            endID = ResID(endChainID, int(endSeqNum), endICode)
            if (first <= initID <= last) and (first <= endID <= last):
                sys.stdout.write(line)

        elif (line_type == "SHEET "):
            initChainID = line[21:22]
            initSeqNum  = int(line[22:26])
            initICode   = line[26:27]
            initID = ResID(initChainID, int(initSeqNum), initICode)
            endChainID  = line[32:33]
            endSeqNum   = int(line[33:37])
            endICode    = line[37:38]
            endID = ResID(endChainID, int(endSeqNum), endICode)
            if (first <= initID <= last) and (first <= endID <= last):
                sys.stdout.write(line)

        elif (line_type == "TURN  "):
            initChainID = line[19:20]
            initSeqNum  = int(line[20:24])
            initICode   = line[24:25]
            initID = ResID(initChainID, int(initSeqNum), initICode)
            endChainID  = line[30:31]
            endSeqNum   = int(line[31:35])
            endICode    = line[35:36]
            endID = ResID(endChainID, int(endSeqNum), endICode)
            if (first <= initID <= last) and (first <= endID <= last):
                sys.stdout.write(line)

        elif line_type == "SEQRES":
            chainID = line[11:12]
            if (first.chainID <= chainID) and (chainID <= last.chainID):
                sys.stdout.write(line)

        elif line_type == "TER   ":
            chainID = line[21:22]
            seqNum  = int(line[22:26])
            iCode   = line[26:27]
            terRes  = ResID(chainID, seqNum, iCode)
            if (first <= resID <= last):
                sys.stdout.write(line)

        else:
            sys.stdout.write(line)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Typical Usage: 

coords2dihedrals.py 180 < 12column_text_file.dat

This script reads a 12-column numeric text file and computes dihedral angles
(and other inclusive angles and distances).  Example input file:

30.13 11.46 15.12 35.28 -2.32 12.61 30.68 -4.45 16.30 28.68 8.46 10.24
28.68 8.42 10.24 30.68 -4.45 16.30 25.22 -4.75 19.42 27.57 3.37 8.06
27.57 3.37 8.06 25.22 -4.75 19.42 20.45 -1.27 20.09 25.80 -2.26 6.61
25.80 -2.23 6.61 20.45 -1.27 20.09 15.47 1.36 18.12 21.67 -5.88 7.17
:
The 12 numbers on each line represent the x,y,z coordinates of 4 atoms.
(This is the format of the files generated by the "pdb2coords.py" script.)
For each line, the program calculates the (4-body) dihedral angle for 
these 4 atoms, followed by the (3-body) bond angle between atoms 1,2,3 and 2,3,4
followed by distances between atoms 1,2 and 2,3 and 3,4, and writes them
(in that order) to the standard out.  (6 numbers total, followed by a newline)
(When a line contains the wrong number of numbers, 
 the script prints out a list of 6 impossibe negative values:
 "-720 -360 -360 -1 -1 -1" to let the caller know that 
 this particular angle could not be computed.)

Note:   
The "IUPAC/IUB" dihedral-angle convention is used:
4 atoms in the "trans" conformation have a dihedral angle of 180 degrees.
By default, dihedral angles are calculated in the range from 0 to 360.0 degrees.
(This means there is discontinuity in the angle at 0 degrees.)
However this may be a bad choice for polymers which can alternate between
helical conformations which are left and right handed.
In some cases, you may want to have the discontinuity appear at 180 degrees.
(or some other angle which is sparsely populated).

So you can supply an optional argument (the "branch_of_log") which indicates 
where the discontinuity in the dihedral angle will appear.
Dihedral angles returned by this program will lie in the range: 
[branch_of_log-360, branch_of_log)

"""

import sys
from math import sqrt, cos, sin, tan, acos, asin, atan, pi, floor
# Sometimes this program pipes its output to other programs which halt early.
# Below we silently suppress the ugly "Broken pipe" message this generates:
import signal
signal.signal(signal.SIGPIPE, signal.SIG_DFL)


def length_v(r):
    lsqd = 0.0
    for d in range(0,len(r)):
        lsqd += r[d]*r[d]
    return sqrt(lsqd)


def inner_prod_v(r1,r2):
    result = 0.0
    for d in range(0,len(r1)):
        result += r1[d]*r2[d]
    return result


def cross_prod_v3(a,b):
    c = [0.0,0.0,0.0]
    c[0] = a[1]*b[2] - a[2]*b[1]
    c[1] = a[2]*b[0] - a[0]*b[2]
    c[2] = a[0]*b[1] - a[1]*b[0]
    return c



def main():

    branch_of_log = 2.0*pi  # by default, dihedral angles lie in range: [0, 360.0)
    truncate_a    = 0
    truncate_b    = 0

    if (len(sys.argv) > 4):
        sys.stderr.write('Error (coords2dihedrals): number of arguments should not exceed 3.\n'\
                         '    If an odd-number of arguments are passed (1 or 3), then\n'
                         '    the first argument is assumed to be the branch-of-log, a number which is\n'
                         '    0 and 360.0.  This argument indicates where the discontinuity in the\n'
                         '    dihedral angle is located.  By default it is 0 degrees, which\n'
                         '    corresponds to 4 atoms in the "cis" conformation.)\n'
                         '   (The two arguments correspond to the number of lines of\n'\
                         '     text to omit from the beginning and end of the file, respectively.)\n'\
                         '     If one argument is passed, then both are assumed to be the same.\n'\
                         '     If no argument is passed, then by default, no data is ignored.\nExiting...\n\n')
        sys.exit(-1)

    # NOTE: The "truncate" arguments are not really supported any more.  Instead
    #       use other scripts to post-process the results printed by this program.
    elif (len(sys.argv) == 4):
        branch_of_log = float(sys.argv[1])
        truncate_a    =   int(sys.argv[2])
        truncate_b    =   int(sys.argv[3])
    elif (len(sys.argv) == 3):
        truncate_a = int(sys.argv[1])
        truncate_b = int(sys.argv[2])
    elif (len(sys.argv) == 2):
        branch_of_log = float(sys.argv[1])
        branch_of_log *= pi/180.0


    coords_list = []
    # Read the file
    for line in sys.stdin:
        line = line.strip()
        # Each line should contain a list of 3 numbers separated by whitespace.
        # If so, store the 3 numbers in a list variable (named xyz), and append
        # it to the list of coordinates.
        # However some lines might also be blank, in which case we append the
        # empty list [] to the list of coordinates.
        if line == '':
            coords = []
        else:
            # Each line should contain a list of 3 numbers separated by whitespace.
            coords = list(map(float, line.split()))
            if len(coords) != 4*3:
                sys.stderr.write('Error(coords2dihedrals):\n'+'Each line should either contain 12 numbers or be blank.\n')
                sys.exit(-1)
        coords_list.append(coords)


    # Truncate the data we don't want.
    # (Why?  The residues at the beginning and ending of helices 
    #  are less trustworthy then the residues in the middle.)
    coords_list = coords_list[truncate_a:len(coords_list)-truncate_b]

    N = len(coords_list)
    for i in range(0,N):
        if len(coords_list[i]) == 3*4:
            r10 = [0.0, 0.0, 0.0]
            r21 = [0.0, 0.0, 0.0]
            r32 = [0.0, 0.0, 0.0]
            for d in range(0,3):
                r10[d] = coords_list[i][3*1+d] - coords_list[i][3*0+d]
                r21[d] = coords_list[i][3*2+d] - coords_list[i][3*1+d]
                r32[d] = coords_list[i][3*3+d] - coords_list[i][3*2+d]
            l10 = length_v(r10)
            l21 = length_v(r21)
            l32 = length_v(r32)

            n012 = cross_prod_v3(r10, r21)
            n123 = cross_prod_v3(r21, r32)

            # The dihedral-angle or 4-body angle is named "angle0124"
            cos_phi = inner_prod_v(n012, n123) /(length_v(n012)*length_v(n123))

            # There is a problem whenever 4 consecutive atoms are coplanar:
            #
            #            *---*
            #                |      (all 4 atoms are coplanar, and phi = 0)
            #            *---*
            #
            # In this example, the dihedral angle phi is well defined and =0.
            # The problem is that, due to floating point roundoff
            # "cos_phi" can sometimes slightly exceed 1.
            # This causes a NAN when you calculate acos(cos_phi).

            if (cos_phi > 1.0):
                cos_phi = 1.0
            elif (cos_phi < -1.0):
                cos_phi = -1.0

            phi = acos(cos_phi)

            # This formula does not distinguish positive and negative phi.
            #
            # Negative dihedral angles:
            #
            # Check if  the position of atom i+3 is above the phi=0 plane
            # (in the region of positive phi), or below the phi=0 plane.
            # It is above the phi=0 plane if the bond from atom i+2 to i+3
            # points in the same direction as the negative-phi-tangent-vector 
            # for atom i (not i+3)  (...which points in the n012 direction)
            if inner_prod_v(n012, r32) < 0.0:
                phi = -phi

            phi_range_a = branch_of_log - (2.0*pi)
            phi_range_b = branch_of_log
            nphi = floor((phi - phi_range_a) / (2.0*pi))
            phi = phi - (nphi*2.0*pi)

            theta0 = length_v(n012) / (l10*l21)
            theta1 = length_v(n123) / (l21*l32)

            sys.stdout.write(str(phi*180.0/pi) + ' ' +
                             str(theta0*180.0/pi) + ' ' +
                             str(theta1*180.0/pi) + ' ' +
                             str(l10) + ' ' +
                             str(l21) + ' ' +
                             str(l32) +
                             '\n')
        else:
            # Otherwise, we write out an impossible values to let the caller 
            # know that this particular dihedral angle could not be computed
            sys.stdout.write('-720 -360 -360 -1 -1 -1\n')


if __name__ == "__main__":
    main()




from math import sqrt, cos, sin, tan, acos, asin, atan, pi

def CalcOmega(theta, phi):
    """
    This function takes two arguments
    theta (the 3-body bond angle,    1st argument)
    phi   (the 4-body torsion angle, 2nd argument)
    ...and returns the "Omega" rotation angle. 

    Omega is the "exterior-angle" made by the poligonal shadow of the backbone
    trace of a helix, when viewed down it's axis of symmetry ("z-axis")
    and projected onto the plane below (the "xy plane").

    For example the text-diagram below, suppose we are viewing a helix
    down it's symmetry axis, and the 4 consecutive alpha-carbon
    atoms leave the following shadow on the plain below.

       *
        \
     *---------------.*     _
          \         _/      /`
           \      _/       /
            \   _/    <-- Omega
             \ /          angle
              *'

    For an alpha helix (which has roughly 3.6 amino-acids per 360-degree turn,
    Omega ~= 360/3.6 ~= 100.0 degrees

    The Omega angle is related to the 3-body bond angle and 4-body torsion
    angle of these 4 atoms (theta and phi).
    All angles (inputs and outputs) are in units of radians.

    """

    Theta = pi - theta
    Phi = pi - phi

    # Derivation:
    #   This comment does not make sense unless 
    #   you also see the figure that goes with it.
    #
    #tan(phi/2) = deltaY / X
    #        X  = cos(a) sin(Omega)
    #   deltaY  = deltay cos(a)
    #    deltay = sin(a) (1-cos(Omega))
    #   -> deltaY = cos(a)sin(a) (1-cos(Omega))
    # -> tan(phi/2) = sin(a)   (1-cos(Omega)) / sin(Omega)
    #               = sin(a) 2 sin^2(Omega/2) / sin(Omega)
    #               = sin(a) 2 sin^2(Omega/2) / 2 sin(Omega/2) cos(Omega/2)
    #               = sin(a) tan(Omega/2)
    #
    #                 cos(a) = sin(Theta/2) / sin(Omega/2)
    #              -> sin(a) = sqrt(1 - sin^2(Theta/2) / sin^2(Omega/2))
    #
    # -> tan(phi/2) = sqrt(1 - sin^2(Theta/2) / sin^2(Omega/2))  tan(Omega/2)
    #               = sqrt(sin^2(Omega/2) - sin^2(Theta/2)) / cos(Omega/2)
    # ->
    #  tan^2(phi/2) = (sin^2(Omega/2) - Sin^2(Theta/2)) / cos^2(Omega/2)
    # We know phi, and Theta.  Now solve for Omega numerically

    tan2phi2 = tan(0.5*phi)
    tan2phi2 *= tan2phi2

    lower_bound = 0.0
    upper_bound = pi
    tolerance = 0.000001
    while ((upper_bound - lower_bound) > tolerance):
        Omega = lower_bound + 0.5*(upper_bound - lower_bound)
        cos2Ome2 =  cos(0.5*Omega)
        cos2Ome2 *= cos2Ome2
        sin2Ome2 =  sin(0.5*Omega)
        sin2Ome2 *= sin2Ome2
        sin2The2 =  sin(0.5*Theta)
        sin2The2 *= sin2The2

        if (sin2Ome2 - sin2The2) / cos2Ome2 > tan2phi2:
            upper_bound = Omega
        else:
            lower_bound = Omega

    # Deal with possible negative values of Omega (left-handed helices):
    if phi < 0.0:
        Omega = -Omega

    return Omega





#!/usr/bin/env python

"""
This program extracts a series of x,y,z coordinates from a PDB file (stdin).
The user must supply a list of 4-character atom names. (Usually these names 
include spaces and thus must be enclosed in quotes when passed to the shell.)

For every residue in the PDB file (which falls within the user's selection),
the coordinates of these atoms are printed to the standard out (stdout),
one line per residue (multiple atoms per line).  The order of the atoms
printed on each line should match the order they were passed as arguments
to this program (not the order they were read from the pdb file).
If any of the requested atom names are not present in the PDB file, then an
"? ? ?" is printed instead at the appropriate location on that line.
(...unless the "-blank" argument is passed, in which case a blank line is 
printed.  Either way the # of output lines equals the # of residues.)

Results can be limited to a specific residue interval in the PDB file.
Intervals are pairs of amino acids signifying the first and last
residue in the interval.  Specifying an interval is optional.
In order to specify any amino acid in a PDB file, you must provide 3
identifiers:
  the ChainID a single letter specifying
  the SeqNum an integer indicating the location within that chain
  the ICode ("insert code" usually 0.  I don't know why this number is 
             necessary. ..For the record, I never loved the PDB file format.)

All 3 identifiers are needed for both the starting and ending residues.
Consequently this program expects 6 arguments: 
   (to be read from the command line)
ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last

 ---  Mixing atoms from different residues together ---
Sometimes bonded interactions occur between atoms in in different 
residues from the same chain.  (For example, the " C  " atom
of one residue binds to the " N  " of the next residue.)
It's convenient to print the coordinates of the atoms involved in these 
interactions together on the same line.  To do that, insert the "i+1"
argument before the " N  " argument, as in:
 " C  "  i+1  " N  "
After the i+1 argument, all of the atoms following it are assumed to come
from the next residue.  (The " N  " atom in this example.)
(Note: If for some reason you want the third atom to come from the original
 residue, you can switch back to it by adding "i+0" after the " N  " atom:
 " C  "  i+1  " N  " i+0 " CA  "
 More generally, you can also use "i+2", or "i+3", ..., and "i-1", "i-2",...
 to mix atoms from more distant residues together.)

Sometimes this program pipes its output to other programs which halt early.
Below we silently suppress the ugly "Broken pipe" message this generates:
"""

# author: Andrew Jewett
g_program_name = __file__.split('/')[-1]
g_date_str = '2017-4-21'
g_version_str = '0.5.1'



import sys
from operator import attrgetter
from math import sqrt
# Sometimes this program pipes its output to other programs which stops reading
# the PDB file prematurely (such as when multiple MODEL records are present).
# Below we silently suppress the ugly "Broken pipe" message this generates:
import signal
signal.signal(signal.SIGPIPE, signal.SIG_DFL)
from resid import *


# Ignore atoms on the backbone (other than CA), 
RAVE_exclude_atoms = [' N  ', ' H  ', ' C  ', ' O  ', ' CA ']
RAVE_exclude_residues = set(['GLY', 'PRO']) #Glycine residues do not have CB atoms


def main():
    atoms_needed = []
    atoms_res_offsets = []
    atoms_res_offset = 0
    omit_incomplete = False
    use_all_residues = True
    firstR = None
    lastR = None

    final_range_a = None
    final_range_b = None
    final_slice_incr = 1



    if len(sys.argv) > 1:
        i = 1
        while i < len(sys.argv):

            if ((sys.argv[i] == '-blank') or (sys.argv[i] == 'blank')):
                omit_incomplete = True
                i += 1

            elif ((sys.argv[i] == 'i') or
                  ((sys.argv[i][:1] == 'i') and (len(sys.argv[i]) >= 3) and
                   (sys.argv[i][1:2] in ['+','-','=']))):
                if sys.argv[i] == 'i':
                    atoms_res_offset = 0
                else:
                    atoms_res_offset = int(sys.argv[i][1:].lstrip('='))
                i += 1

            elif ((len(sys.argv[i]) >= 2) and
                  (sys.argv[i][0] == '[') and (sys.argv[i][-1] == ']')):

                tokens=(sys.argv[i][1:-1]).split(':')
                if len(tokens[0].strip()) > 0:
                    final_range_a = int(tokens[0])
                if len(tokens[1].strip()) > 0:
                    final_range_b = int(tokens[1])
                if len(tokens[2].strip()) > 0:
                    final_slice_incr = int(tokens[2])
                i += 1

            elif (len(sys.argv[i]) == 4):
                # Keep track of the atom type name and the order it appeared
                atoms_needed.append(sys.argv[i])
                # Keep track of from which residue it comes from
                atoms_res_offsets.append(atoms_res_offset)
                i += 1

            else: # if not is_digit(sys.argv[i][0])

                # If the next argument is a number, then interpret this number
                # as a residue sequence number
                if len(sys.argv) < i+6:
                    sys.stderr.write("Error: Not enough arguments or argument type error:\n"
                                     "       Offending arguemt #"+str(i)+": \""+sys.argv[i]+"\"\n")

                    if len(sys.argv[i]) == 1:
                        sys.stderr.write("\n"
                                         "       Is argument \""+sys.argv[i]+"\" a chain ID letter?\n"
                                         "       (Chain IDs are only passed as arguments when you want to limit the\n"
                                         "        residues considered to within an interval in the PDB file.\n"
                                         "        To specify an interval, you must provide 5 more aruments.  See below.)\n"
                                         "\n"
                                         "       Note: one-letter residue codes (eg A,V,L,...) are not considered\n"
                                         "             valid residue types by this program.\n"
                                         "             You must specify a 3-letter equivalent (eg ALA, VAL, LYS,...).\n"
                                         "             One-letter arguments are interpreted as chain-IDs. (See below.)\n"
                                         "       \n"
                                         "       By default, the the sequence is extracted from the entire PDB file.\n"
                                         "       In that case, no arguments are required.\n"
                                         "       Alternately, you can limit the selection to a single interval of\n"
                                         "       residues from one of the chains in the PDB file.\n"
                                         "       To specify an interval, you must passing 6 arguments to this program.\n"
                                         "       This program requires a pair of residues to designate the first and\n"
                                         "       last members of the interval.  Each residue requires 3 identifiers.\n"
                                         "       Consequently the six arguments needed are:\n"
                                         "\n"
                                         "ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last\n"
                                         "\n")
                    else:
                        sys.stderr.write("\n"
                                         "Note: Atom-type names should be exactly 4 characters long, and typically\n"
                                         "      contain spaces.  For example: \" C  \", \" O2 \", or \" C1\'\"\n"
                                         "\n"
                                         "Note: PDB files are not passed as arguments\n"
                                         "      but are read from the standard input, for example using the notation\n"
                                         "\n"
                                         "      "+sys.argv[0]+" < file.pdb\n")
                    exit(-1)
                else:
                    use_all_residues = False
                    firstR = ResID(sys.argv[i], int(sys.argv[i+1]), sys.argv[i+2])
                    lastR  = ResID(sys.argv[i+3], int(sys.argv[i+4]), sys.argv[i+5])
                    #sys.stderr.write('  Interval selected: (\"'+firstR.chainID+'\", '+str(firstR.seqNum)+', \"'+firstR.iCode+'\") ... (\"'+lastR.chainID+'\", '+str(lastR.seqNum)+', \"'+lastR.iCode+'\")\n')
                    i += 6

    resID2coords   = {}
    resID2CrdTot   = {}
    resID2CrdSqTot = {}
    resID2CrdNum   = {}
    model_ID = None

    for line in sys.stdin:
        #print("\""+line.rstrip()+"\"")
        if line[0:6] == "MODEL ":
            if model_ID == None:
                model_ID = line[10:14]
            else:
                # Otherwise, if it's not the first model quit
                sys.stderr.write('  Warning(pdb2coords.py): Omitted alternate models from pdb file.\n')
                break
        elif line[0:6] == "ATOM  ":
            #atomID    = int(line[6:11])
            atomType  = line[12:16]
            altLoc    = line[16:17]
            resType   = line[17:20]
            chainID   = line[21:22]
            resSeq    = line[22:26]
            iCode     = line[26:27]
            resID = ResID(chainID, int(resSeq), iCode)
            #xyz_str   = line[30:54]<-not safe. spaces not always in between numbers
            x_str     = line[30:38].strip()
            y_str     = line[38:46].strip()
            z_str     = line[46:54].strip()
            if (use_all_residues or ((firstR<=resID) and (resID<=lastR))):
                if resID not in resID2coords:
                    resID2coords[resID]   = [None for i in range(0,len(atoms_needed))]
                    resID2CrdTot[resID]   = [0.0, 0.0, 0.0]
                    resID2CrdSqTot[resID] = [0.0, 0.0, 0.0]
                    resID2CrdNum[resID]   = 0

                if altLoc == ' ':  # (Ignore "alternate" atom records)
                    for i in range(0, len(atoms_needed)):

                        if atomType == atoms_needed[i]:
                            resID2coords[resID][i] = [x_str, y_str, z_str]

                        if ((not (atomType in RAVE_exclude_atoms)) and
                            (not (resType in RAVE_exclude_residues))):
                            resID2CrdNum[resID] += 1
                            crd_tot = resID2CrdTot[resID]
                            crd_tot[0] += float(x_str)
                            crd_tot[1] += float(y_str)
                            crd_tot[2] += float(z_str)
                            crd_sq_tot = resID2CrdSqTot[resID]
                            crd_sq_tot[0] += float(x_str)**2
                            crd_sq_tot[1] += float(y_str)**2
                            crd_sq_tot[2] += float(z_str)**2



    # Extract an (unordered) list of the resIDs of the residues in the sequence
    resIDs = [resID for resID in resID2coords]

    # Residues in PDB files are often not listed in order.
    # Consequently, we must sort the list by chainID, seqNum, and finnaly iCode:
    sequence_of_resIDs = sorted(resIDs, key=attrgetter('chainID','seqNum','iCode'))

    positions        = [ [None for j in range(0,len(atoms_needed))] 
                         for i in range(0,len(sequence_of_resIDs)) ]

    assert(len(atoms_needed) == len(atoms_res_offsets))

    if len(atoms_needed) > 0:
        min_offset = None
        max_offset = None
        for offset in atoms_res_offsets:
            if ((min_offset == None) or (min_offset > offset)):
                min_offset = offset
            if ((max_offset == None) or (max_offset < offset)):
                max_offset = offset
    else:
        sys.stderr.write('  Warning(pdb2coords.py): NO ATOM TYPES SELECTED.\n')
        min_offset = 0
        max_offset = 0

    # If the user requested any offsets (so that some atoms on the same line 
    # of output belong to different residues), then this means that by
    # default (if omit_incomplete is False), additional lines of coordinate
    # data will be printed which contain references to missing coordinates
    # (because some of the needed atoms are out of index range).
    # The "num_extra" variable equals the number the number of additional lines
    # of output.
    num_extra = max_offset - min_offset
    N = len(sequence_of_resIDs)

    for i in range(0, len(atoms_res_offsets)):
        atoms_res_offsets[i] -= min_offset

    sorted_positions = [ [None for j in range(0,len(atoms_needed))] 
                         for i in range(0, N+num_extra) ]

    for i in range(0, N):
        resID = sequence_of_resIDs[i]
        positions[i] = resID2coords[resID]

        # now deal with atoms of type "RAVE"
        for j in range(0, len(atoms_needed)):
            if atoms_needed[j] == 'RAVE':
                RAVE_num_atoms = resID2CrdNum[resID]
                if RAVE_num_atoms > 0:
                    positions[i][j] = [str(resID2CrdTot[resID][0] / RAVE_num_atoms),
                                       str(resID2CrdTot[resID][1] / RAVE_num_atoms),
                                       str(resID2CrdTot[resID][2] / RAVE_num_atoms)]
                else:
                    assert(positions[i][j] == None)
            elif atoms_needed[j] == 'RGYR':
                RAVE_num_atoms = resID2CrdNum[resID]
                if RAVE_num_atoms > 0:
                    rave   = ((resID2CrdTot[resID][0] / RAVE_num_atoms),
                              (resID2CrdTot[resID][1] / RAVE_num_atoms),
                              (resID2CrdTot[resID][2] / RAVE_num_atoms))
                    rsqave = ((resID2CrdSqTot[resID][0] / RAVE_num_atoms),
                              (resID2CrdSqTot[resID][1] / RAVE_num_atoms),
                              (resID2CrdSqTot[resID][2] / RAVE_num_atoms))
                    rgyrsq = 0.0
                    for d in range(0,3):
                        rgyrsq += rsqave[d] - (rave[d]*rave[d])
                    rgyr = sqrt(rgyrsq)
                    positions[i][j] = [str(rgyr), '', '']
                    if rgyr == 0.0:
                        print('resID=\"'+str(resID)+'\"')
                        exit(0)
                else:
                    assert(positions[i][j] == None)


    for i in range(-num_extra, N):
        for j in range(0,len(atoms_needed)):
            I = i + atoms_res_offsets[j]
            assert(I >= i)
            if (0 <= I) and (I < N):
                sorted_positions[i+num_extra][j] = positions[I][j]

    if final_range_a == None:
        final_range_a = 0
    else:
        final_range_a += max_offset

    if final_range_b == None:
        final_range_b = N+num_extra
    else:
        final_range_b += max_offset

    for i in range(final_range_a, final_range_b, final_slice_incr):
        coords_str_list = []
        for j in range(0,len(sorted_positions[i])):
            if sorted_positions[i][j] == None:
                if omit_incomplete:
                    coords_str_list = []
                    break
                else:
                    coords_str_list += ['?','?','?']
            else:
                coords_str_list += sorted_positions[i][j]

        # Finally, write out the coordinates.
        sys.stdout.write(' '.join(coords_str_list) + '\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program reads in x,y,z coordinates from a file
and computes the "helixAngleOmega"
(defined in "helix_Omega_angle_derivation.png" in the "doc" subdirectory)
This has been used to infer the periodicity of alpha-helices within proteins.
"""


from math import sqrt, cos, sin, tan, acos, asin, atan, pi
import sys
from helixAngleOmega import CalcOmega



def length_v(r):
    lsqd = 0.0
    for d in range(0,len(r)):
        lsqd += r[d]*r[d]
    return sqrt(lsqd)


def inner_prod_v(r1,r2):
    result = 0.0
    for d in range(0,len(r1)):
        result += r1[d]*r2[d]
    return result


def cross_prod_v3(a,b):
    c = [0.0,0.0,0.0]
    c[0] = a[1]*b[2] - a[2]*b[1]
    c[1] = a[2]*b[0] - a[0]*b[2]
    c[2] = a[0]*b[1] - a[1]*b[0]
    return c


def main():
    if (len(sys.argv) > 3):
        sys.stderr.write('Error (coords2angles_Omega): number of arguments should not exceed 2.\n'\
                             '    (The two arguments correspond to the number of lines of\n'\
                             '     text to omit from the beginning and end of the file, respectively.)\n'\
                             '     If one argument is passed, then both are assumed to be the same.\n'\
                             '     If no argument is passed, then by default, no data is ignored.\nExiting...\n\n')
        sys.exit(-1)

    elif (len(sys.argv) == 3):
        truncate_a = int(sys.argv[1])
        truncate_b = int(sys.argv[2])
    elif (len(sys.argv) == 2):
        truncate_a = int(sys.argv[1])
        truncate_b = truncate_a
    else:
        truncate_a = 0
        truncate_b = 0


    r_i = []
    # Read the file
    for line in sys.stdin:
        line = line.strip()
        # Each line should contain a list of 3 numbers separated by whitespace.
        # If so, store the 3 numbers in a list variable (named xyz), and append
        # it to the list of coordinates.
        # However some lines might also be blank, in which case we append the
        # empty list [] to the list of coordinates.
        if line == '':
            xyz = []
        else:
            # Each line should contain a list of 3 numbers separated by whitespace.
            xyz = list(map(float, line.split()))
        r_i.append(xyz)


    # Truncate the data we don't want.
    # (Why?  The residues at the beginning and ending of helices 
    #  are less trustworthy then the residues in the middle.)
    r_i = r_i[truncate_a:len(r_i)-truncate_b]

    N = len(r_i)
    for i in range(0,N-4+1):

        if (len(r_i[i])==3)and(len(r_i[i+1])==3)and(len(r_i[i+2])==3)and(len(r_i[i+3])==3):

            r10 = [0.0, 0.0, 0.0]
            r21 = [0.0, 0.0, 0.0]
            r32 = [0.0, 0.0, 0.0]
            for d in range(0,3):
                r10[d] = r_i[i+1][d] - r_i[i][d]
                r21[d] = r_i[i+2][d] - r_i[i+1][d]
                r32[d] = r_i[i+3][d] - r_i[i+2][d]
            l10 = length_v(r10)
            l21 = length_v(r21)
            l32 = length_v(r32)

            n012 = cross_prod_v3(r10, r21)
            n123 = cross_prod_v3(r21, r32)


            # The torsion-angle or 4-body angle is named "angle0124"
            cos_phi = inner_prod_v(n012, n123) /(length_v(n012)*length_v(n123))

            # There is a problem whenever 4 consecutive atoms are coplanar:
            #
            #            *---*
            #                |      (all 4 atoms are coplanar, and phi = 0)
            #            *---*
            #
            # In this example, the torsion angle phi is well defined and =0.
            # The problem is that, due to floating point roundoff
            # "cos_phi" can sometimes slightly exceed 1.
            # This causes a NAN when you calculate acos(cos_phi).

            if (cos_phi > 1.0):
                cos_phi = 1.0
            elif (cos_phi < -1.0):
                cos_phi = -1.0

            phi = acos(cos_phi)

            # This formula does not distinguish positive and negative phi.
            #
            # Negative torsion angles:
            #
            # Check if  the position of atom i+3 is above the phi=0 plane
            # (in the region of positive phi), or below the phi=0 plane.
            # It is above the phi=0 plane if the bond from atom i+2 to i+3
            # points in the same direction as the negative-phi-tangent-vector 
            # for atom i (not i+3)  (...which points in the n012 direction)
            if inner_prod_v(n012, r32) < 0.0:
                phi = -phi

            # The two bond-angles or 3-body angles are named "angle012" and "angle123"
            angle012 = acos( -inner_prod_v(r10, r21) / (l10 * l21) )
            angle123 = acos( -inner_prod_v(r21, r32) / (l21 * l32) )
            # (The negative sign above comes from the fact that we are really 
            #  interested in the angle between r21 and r01 (which is -r10).)

            # Convert these angles to degrees, and print it out
            #sys.stdout.write(str(angle012*180.0/pi)+' ')
            #sys.stdout.write(str(angle123*180.0/pi)+' ')

            # Let theta = the average of the two 3-body angles
            theta = 0.5 * (angle012 + angle123) 

            # Omega (usually 360/3.6 ~= 100 degrees) is the helix rotation angle.
            Omega = CalcOmega(theta, phi)

            sys.stdout.write(str(Omega*180.0/pi))

        else:
            # Otherwise, we write out an impossible value (-720) to let the caller 
            # know that this particular angle could not be computed
            sys.stdout.write('-720')

        if i+1 < N-4+1:
            sys.stdout.write(' ')

    sys.stdout.write('\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Usage:

coords2angles.py < 9column_text_file.dat

This script reads a 9-column numeric text file.  For example:

30.13 11.46 15.12 35.28 -2.32 12.61 30.68 -4.45 16.30
28.68 8.42 10.24 30.68 -4.45 16.30 25.22 -4.75 19.42
27.57 3.37 8.06 25.22 -4.75 19.42 20.45 -1.27 20.09
25.80 -2.23 6.61 20.45 -1.27 20.09 15.47 1.36 18.12
:
The 9 numbers on each line represent the x,y,z coordinates of 3 atoms.
(This is the format of the files generated by the "pdb2coords.py" script.)
For each line, the program calculates the angle between these atoms
followed by distances between atoms 1,2 and 2,3 (in that order)
to the standard out.  (3 numbers total, followed by a newline.)
(When a line contains the wrong number of numbers, 
 the script prints out a list of 3 impossibe negative values: "-360 -1 -1"
 to let the caller know that this particular angle could not be computed

"""

import sys
from math import sqrt, cos, sin, tan, acos, asin, atan, pi
# Sometimes this program pipes its output to other programs which halt early.
# Below we silently suppress the ugly "Broken pipe" message this generates:
import signal
signal.signal(signal.SIGPIPE, signal.SIG_DFL)


def length_v(r):
    lsqd = 0.0
    for d in range(0,len(r)):
        lsqd += r[d]*r[d]
    return sqrt(lsqd)


def inner_prod_v(r1,r2):
    result = 0.0
    for d in range(0,len(r1)):
        result += r1[d]*r2[d]
    return result


def main():
    if (len(sys.argv) > 3):
        sys.stderr.write('Error (coords2angles): number of arguments should not exceed 2.\n'\
                          '    (The two arguments correspond to the number of lines of\n'\
                          '     text to omit from the beginning and end of the file, respectively.)\n'\
                          '     If one argument is passed, then both are assumed to be the same.\n'\
                          '     If no argument is passed, then by default, no data is ignored.\nExiting...\n\n')
        sys.exit(-1)

    # NOTE: The "truncate" arguments are not really supported any more.  Instead
    #       use other scripts to post-process the results printed by this program.
    elif (len(sys.argv) == 3):
        truncate_a = int(sys.argv[1])
        truncate_b = int(sys.argv[2])
    elif (len(sys.argv) == 2):
        truncate_a = int(sys.argv[1])
        truncate_b = truncate_a
    else:
        truncate_a = 0
        truncate_b = 0


    coords_list = []
    # Read the file
    for line in sys.stdin:
        line = line.strip()
        if line == '':
            coords = []
        else:
            # Each line should contain a list of 9 numbers separated by whitespace.
            coords = list(map(float, line.split()))
            if len(coords) != 3*3:
                sys.stderr.write('Error(coords2angles.py):\n'+'Each line should either contain 9 numbers or be blank.\n')
                sys.exit(-1)
        coords_list.append(coords)


    # Truncate the data we don't want.
    # (Why?  The residues at the beginning and ending of helices 
    #  are less trustworthy then the residues in the middle.)
    coords_list = coords_list[truncate_a:len(coords_list)-truncate_b]

    N = len(coords_list)
    for i in range(0,N):
        if len(coords_list[i]) == 3*3:
            r10 = [0.0, 0.0, 0.0]
            r21 = [0.0, 0.0, 0.0]
            for d in range(0,3):
                r10[d] = coords_list[i][3*1+d] - coords_list[i][3*0+d]
                r21[d] = coords_list[i][3*2+d] - coords_list[i][3*1+d]
            l10 = length_v(r10)
            l21 = length_v(r21)

            # The bond angle is the angle between r10 and r21
            cos_theta = -inner_prod_v(r10, r21) /(length_v(r10)*length_v(r21))

            if (cos_theta > 1.0):
                cos_theta = 1.0
            elif (cos_theta < -1.0):
                cos_theta = -1.0

            theta = acos(cos_theta)

            sys.stdout.write(str(theta*180.0/pi)+" "+str(l10)+" "+str(l21)+'\n')

        else:
            # Otherwise, we write out impossible values to let the caller 
            # know that this particular angle could not be computed
            sys.stdout.write('-360 -1 -1\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
 Typical usage (example):

    dlpdb.py < list_of_pdb_codes.txt

Where the input file contains a list of pdb-codes, one per line:
100d
101d
102d
103d
104d
107d
108d
109d
 Every line in this file contains a 4-letter pdb identifier.
Each 4-letter pdb code is compared with a list of pdb codes downloaded so far
(which is initially read from the file "pdbs_most_recent.txt", if present).
If it is new 4-letter pdb code, then the corresponding pdb file is downloaded,
and appended to the end of the "pdbs_most_recent.txt" file.
The corresponding DSSP file is also downloaded, if available.

"""

# author: Andrew Jewett
g_program_name = __file__.split('/')[-1]
g_date_str = '2017-4-21'
g_version_str = '0.5.1'


import sys, urllib2, random, time, gzip


def FileExists(fname):
    try:
        file = open(fname, 'r')
    except IOError:
        exists = False
    else:
        exists = True
    return exists



def ExtractFileName(url):
    # The name of the file we create will be the same as the last few 
    # characters of the URL. (Whatever comes after the last '/' character.)
    out_fname = url
    slash_loc = out_fname.rfind('/')
    if (slash_loc >= 0):  # if '/' was found, then remove text preceeding '/'
        out_fname = out_fname[slash_loc+1:]
        if (out_fname == ''):
            sys.stderr.write('Error: filename difficult to determine from URL:\n' +\
                                 '       \"'+url+'\"\n')
            sys.exit(-1)
    return out_fname



# This function downloads a file from a URL, and (if requested by the user)
# prints out a warning message if there was a problem.

import os

def DownloadFileTo(url, file_name, verbose_mode = True):
    # Open the input file (at url) for reading
    # Simple way:
    #   in_file = urllib.urlopen(url)
    # But the web site may return garbage if the file doesn't exist.
    # Instead, we check for errors first:
    request = urllib2.Request(url)
    try: in_file = urllib2.urlopen(request)
    except urllib2.URLError, e:
        if (verbose_mode):
            sys.stderr.write(str(e)+'\n')
            sys.stderr.write('   omitting file \"'+file_name+'\"\n')
    else:
        if (verbose_mode):
            sys.stderr.write('downloading file \"'+file_name+'\"\n')
        # Open the output file for writing
        out_file = open(file_name, 'wb')
        out_file.write(in_file.read())
        out_file.close()



# This version infers the file name from the URL.
def DownloadFile(url, verbose_mode = True):
    DownloadFileTo(url, ExtractFileName(url), verbose_mode)


def main():

    # Read in the list of PDB files already downloaded:

    pdbs_old = set([])
    try:
        pdbs_old_file = open('pdbs_old.txt', 'r')
        for pdb_code in pdbs_old_file:
            pdb_code = pdb_code.strip()#eliminate trailing and preceeding whitespace
            pdb_code = pdb_code.lower()#pdb codes are case-insensitive
            if (len(pdb_code) != 4):
                sys.stderr.write('Error in in \"pdbs_old.txt\":\n')
                sys.stderr.write('      Invalid PDB-code: \"'+pdb_code+'\"\n')
                exit(-1)
            pdbs_old.add(pdb_code)
        pdbs_old_file.close()
    except IOError:
        pass




    # Here we keep track of the list of pdb files which
    #  i) are in the current list of pdb files requested in 
    #     sys.stdin
    # ii) are in the current list, but are not downloaded yet 
    #     (new pdb files)
    pdbs_current_file = open('pdbs_most_recent.txt', 'wb')
    pdbs_current = set([]) #entire list of pdb codes requested
    pdbs_new  = set([]) #a list of new pdb codes requested that were not in the old list
    pdbs_old_file = open('pdbs_old.txt', 'a')


    for line in sys.stdin:
        line = line.strip()
        if len(line) == 0:
            continue
        pdb_code = line.lower()  #(pdb codes are case-insensitive)

        if (pdb_code in pdbs_new):
            sys.stderr.write(pdb_code+' appears redundantly. skipping\n')
        elif (pdb_code in pdbs_old):
            if (not (pdb_code in pdbs_current)):
                sys.stderr.write(pdb_code+' downloaded already. skipping.\n')
                pdbs_current.add(pdb_code)
                pdbs_current_file.write(pdb_code+'\n')
                pdbs_current_file.flush()  #<- necessary in case we get interrupted
            else:
                sys.stderr.write(pdb_code+' appears redundantly. skipping\n')

        else:
            #Download the corresponding pdb file
            file_name = pdb_code+'.pdb.gz' # <- these are compressed files
            url = 'http://www.rcsb.org/pdb/files/'+file_name
            # Note, if the URL above files, try this one instead:
            # url = 'https://files.rcsb.org/download/'+file_name
            DownloadFileTo(url, file_name)

            #Unzip the pdb file:
            with gzip.open(file_name, 'rb') as f:
                file_content = f.read(f)
                f.close()
            f = open(pdb_code+'.pdb', 'wb')
            f.write(file_content)
            if not FileExists(pdb_code+'.pdb'):
                sys.stderr.write('Error: A problem occured when trying to download PDB code \"'+pdb_code+'\"\n'
                                 '       Delete this entry from the file \"pdbs_old.txt\", and rerun '+g_program_name+'\n')
                sys.exit(-1)

            #Optional: Download the corresponding DSSP file
            url = 'ftp://ftp.cmbi.ru.nl/pub/molbio/data/dssp/'+pdb_code+'.dssp'
            DownloadFile(url)
            if not FileExists(pdb_code+'.dssp'):
                sys.stderr.write("    (The old DSSP PDB is server flaking out again.\n"
                                 "     Don't worry.  DSSP files are not needed.)\n")

            #Keep track of the the pdbs we have downloaded so far:

            pdbs_current.add(pdb_code)
            pdbs_current_file.write(pdb_code+'\n')
            pdbs_current_file.flush()  #<- necessary in case we get interrupted
            pdbs_old_file.write(pdb_code+'\n')
            pdbs_old_file.flush()      #<- necessary in case we get interrupted
            pdbs_new.add(pdb_code)


    pdbs_current_file.close()

    def ChainFromPDBfile(chainID, pdb_file):
        for line in pdb_file:
            line_type = line[0:6]
            if line_type in set(['ATOM  ', 'HETATM', 'ANISOU', 'SIGATM', 'SIGUIJ']):
                if line[21:22] == chainID:
                    yield line
            elif line_type == "HET   ":
                if line[12:13] == chainID:
                    yield line
            elif line_type == "HELIX ":
                initChainID = line[19:20]
                endChainID  = line[31:32]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "SHEET ":
                initChainID = line[21:22]
                endChainID  = line[32:33]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "TURN  ":
                initChainID = line[19:20]
                endChainID  = line[30:31]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "SEQRES":
                if line[11:12] == chainID:
                    yield line
            elif line_type == "TER   ":
                ter_chainID = line[21:22]
                if (ter_chainID == chainID) or (ter_chainID == " "):
                    yield line
            else:
                yield line
        return


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
 Usage:

    pdb2sequence.py < PDB_FILE

 To select a limited
    pdb2sequence.py ChainID1 SeqNum1 ICode1 ChainID2 SeqNum2 ICode2 < PDB_FILE

 This program extracts the sequence of residue types from a PDB file, 
 and prints it the the standard out (all on one line).
 Each residue is converted to a 1-letter amino-acid code.
 The 20 standard amino-acids, and 5 nucleic acid types are supported.
 Unknown residue types are denoted 'x'.
 (To add additional residue types, edit the code and add entries to the
 "three2one" lookup table.)

 WARNING: Residues represented as HETATM records will be ignored (skipped)!

     Optional interval arguments:

 Users can limit the PDB file to residues which lie within an interval.

 Intervals are pairs of amino acids signifying the first and last
 residue in the interval.
 In order to specify any amino acid in a PDB file, you must provide 3
 identifiers:
   the ChainID a single letter specifying
   the SeqNum an integer indicating the location within that chain
   the ICode ("insert code" usually 0.  I don't know why this number is 
              necessary. ..For the record, I never loved the PDB file format.
              BUT THIS PROGRAM EXPECTS AN ICode, SO YOU MUST SUPPLY ONE.)
 All 3 identifiers are needed for both the starting and ending residues.
   ---->
 SO, this program expects either 0 or 6 arguments: 
    (to be read from the command line)
 ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last
"""

import sys
from operator import attrgetter
from resid import *


def main():
    if len(sys.argv) == 1:
        use_all_residues = True
    elif len(sys.argv) == 7:
        use_all_residues = False
        first = ResID(sys.argv[1], int(sys.argv[2]), sys.argv[3])
        last  = ResID(sys.argv[4], int(sys.argv[5]), sys.argv[6])
    else:
        sys.stderr.write("Error: This program requires either 0 or 6 arguments.\n"
                         "       By default, the the sequence is extracted from the entire PDB file.\n"
                         "       In that case, no arguments are required.\n"
                         "       Alternately, you can limit the selection to a single interval of\n"
                         "       residues from one of the chains in the PDB file.\n"
                         "       To specify an interval, you must passing 6 arguments to this program.\n"
                         "       This program requires a pair of residues to designate the first and\n"
                         "       last members of the interval.  Each residue requires 3 identifiers.\n"
                         "       Consequently the six arguments needed are:\n"
                         "ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last\n")
        exit(-1)

    three2one = {\
        'GLY':'G',\
        'ALA':'A',\
        'SER':'S',\
        'CYS':'C',\
        'VAL':'V',\
            \
        'THR':'T',\
        'ILE':'I',\
        'PRO':'P',\
        'MET':'M',\
        'ASP':'D',\
            \
        'ASN':'N',\
        'LEU':'L',\
        'LYS':'K',\
        'GLU':'E',\
        'GLN':'Q',\
            \
        'ARG':'R',\
        'HIS':'H',\
        'PHE':'F',\
        'TYR':'Y',\
        'TRP':'W',
            \
        ' DA':'A',\
        ' DC':'C',\
        ' DG':'G',\
        ' DT':'T',\
            \
        '  A':'A',\
        '  U':'U', \
        '  G':'G',\
        '  C':'C'}



    resID2type3 = {}

    for line in sys.stdin:
        #if line[0:6] in ("ATOM  ", "HETATM"):
        if line[0:6] in "ATOM  ":
            #atomID    = int(line[6:11])
            #atomType  = line[12:16]
            #altLoc    = line[16:17]
            resType   = line[17:20]
            chainID   = line[21:22]
            resSeq    = line[22:26]
            iCode     = line[26:27]
            resID = ResID(chainID, int(resSeq), iCode)
            if (use_all_residues or ((first<=resID) and (resID<=last))):
                resID2type3[resID] = resType

    # Extract an (unordered) list of the resIDs of the residues in the sequence
    resIDs = [resID for resID in resID2type3]

    # Residues in PDB files are often not listed in order.
    # Consequently, we must sort the list by chainID, seqNum, and finnaly iCode:
    sequence_of_resIDs = sorted(resIDs, key=attrgetter('chainID','seqNum','iCode'))


    # Now loop through the sequence of resIDs, and 
    # lookup the residue type for each residue (ie. 'ALA', 'PRO, 'GLY', 'VAL' etc...)
    # and convert it to a 1-letter residue name ('A', 'P', 'G', 'V'...)
    # and store the sequence in a string 'APGV...'

    sequence = ''
    for resID in sequence_of_resIDs:
        three_letter_code = resID2type3[resID]
        if three_letter_code in three2one:
            c = three2one[three_letter_code]
        else:
            c = 'x' # default character used for unknown/non-standard residues
        sequence += c

    # Now, finally print out the sequence:
    sys.stdout.write(sequence+'\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program reads a 6-column numeric text file and prints a list of distances.
The 6 numbers on each line represent the x,y,z coordinates of 2 atoms.
For each line this program computes the distance between these atom pairs,
and prints it to the standard out.
(When a line contains the wrong number of numbers, this program does not crash.
 Instead an impossible value, -1.0 is printed.)

"""


import sys
from math import sqrt
# Sometimes this program pipes its output to other programs which halt early.
# Below we silently suppress the ugly "Broken pipe" message this generates:
import signal
signal.signal(signal.SIGPIPE, signal.SIG_DFL)


def length_v(r):
    lsqd = 0.0
    for d in range(0,len(r)):
        lsqd += r[d]*r[d]
    return sqrt(lsqd)


def main():
    if (len(sys.argv) > 3):
        sys.stderr.write('Error (coords2distances): number of arguments should not exceed 2.\n'\
                          '    (The two arguments correspond to the number of lines of\n'\
                          '     text to omit from the beginning and end of the file, respectively.)\n'\
                          '     If one argument is passed, then both are assumed to be the same.\n'\
                          '     If no argument is passed, then by default, no data is ignored.\nExiting...\n\n')
        sys.exit(-1)

    # NOTE: The "truncate" arguments are not really supported any more.  Instead
    #       use other scripts to post-process the results printed by this program.
    elif (len(sys.argv) == 3):
        truncate_a = int(sys.argv[1])
        truncate_b = int(sys.argv[2])
    elif (len(sys.argv) == 2):
        truncate_a = int(sys.argv[1])
        truncate_b = truncate_a
    else:
        truncate_a = 0
        truncate_b = 0


    coords_list = []
    # Read the file
    for line in sys.stdin:
        line = line.strip()
        # Each line should contain a list of 6 numbers separated by whitespace.
        # If so, store the 6 numbers in a list variable (named xyz), and append
        # it to the list of coordinates.
        # However some lines might also be blank, in which case we append the
        # empty list [] to the list of coordinates.
        if line == '':
            coords = []
        else:
            # Each line should contain a list of 3 numbers separated by whitespace.
            coords = list(map(float, line.split()))
            if len(coords) != 2*3:
                sys.stderr.write('Error(coords2distances.py):\n'+'Each line should either contain 6 numbers or be blank.\n')
                sys.exit(-1)
        coords_list.append(coords)


    # Truncate the data we don't want.
    # (Why?  The residues at the beginning and ending of helices 
    #  are less trustworthy then the residues in the middle.)
    coords_list = coords_list[truncate_a:len(coords_list)-truncate_b]

    N = len(coords_list)
    for i in range(0,N):
        if len(coords_list[i]) == 2*3:
            r10 = [0.0, 0.0, 0.0]
            for d in range(0,3):
                r10[d] = coords_list[i][3*1+d] - coords_list[i][3*0+d]
            l10 = length_v(r10)

            sys.stdout.write(str(l10)+'\n')

        else:
            # Otherwise, we write out an impossible value (-1.0) to let the caller 
            # know that this particular distance could not be computed
            sys.stdout.write('-1.0\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program currently does not do anything which grep could not do. 
This returns prints nothing, but it returns (using an exit code)
whether or not a PDB file contains any secondary structure 
information. If it doesn't it usually means that the original authors of 
the PDB file did not indicate where the they are.

This program was not meant to be invoked explicitly by the user.
(In the future, perhaps I will eliminate it.)
"""

import sys

def main():
    secondary_str_found = False;
    for line in sys.stdin:
        if ((line[0:6] == "HELIX ") or
            (line[0:6] == "SHEET ") or
            (line[0:5] == "TURN ")):
            secondary_str_found = True;

    if (secondary_str_found):
        exit(0) #normal termination indicates a helix/sheet/turn was found
    else:
        exit(1) #non-zero (abnormal termination) exit code indicates this
                #PDB file does not contain any secondary structure information

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Truncates the first and last a,b tokens from each line,
where a,b are arguments from sys.argv.
(The truncate_char.py on the other hand, truncates individual characters)
Strings are split into "tokens" use white-space as a delimiter.
"""

import sys


def main():

    if (len(sys.argv) <= 1):
        sys.stderr.write('Error: expected the number of truncations as an argument\n')
    truncate_a = int(sys.argv[1])
    if (len(sys.argv) >= 3):
        truncate_b = int(sys.argv[2])
    else:
        truncate_b = truncate_a

    for line in sys.stdin:
        line = line.strip()
        tokens = line.split()
        if (len(line) > (truncate_a + truncate_b)):
            truncated_tokens = tokens[truncate_a:len(tokens)-truncate_b]
        else:
            truncated_tokens = []
        for i in range(0,len(truncated_tokens)):
            sys.stdout.write(truncated_tokens[i])
            if i+1 < len(truncated_tokens):
                sys.stdout.write(' ')
        sys.stdout.write('\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program finds the location of each HELIX in a PDB file.
(For each helix, it prints out the starting and ending residue
 using a format which is recognized by the "select_interval.py" script.
 That script is typically used for extracting excerpts of a PDB file.)
This script is not really intended to be used on it's own.
"""

import sys

def main():
    for line in sys.stdin:
        if (line[0:6] == "HELIX "):
            initChainID = line[19:20]
            initSeqNum  = int(line[21:25])
            initICode   = line[25:26]
            endChainID  = line[31:32]
            endSeqNum   = int(line[33:37])
            endICode    = line[37:38]
            sys.stdout.write("\""+initChainID+"\" "+str(initSeqNum)+" \""+initICode+
                             "\"  \""+
                             endChainID+"\" "+str(endSeqNum)+" \""+endICode+"\"\n")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
 Usage:

   dssp2pdb.py PDB_FILE < DSSP_FILE > helix_sheet_records.pdb

 This program reads a DSSP file, and generates lines of text which can
 be inserted into a PDB file.  These lines of text are
 "HELIX" and "SHEET" records, which can be interpreted by other programs.

 Information on the DSSP format was obtained from:
 http://swift.cmbi.ru.nl/gv/dssp/DSSP_3.html

 DSSP files for proteins with 4-letter PDB/RCSB identifiers
 can be downloaded from
 ftp://ftp.cmbi.kun.nl/pub/molbio/data/dssp/????.dssp
 For example:
 ftp://ftp.cmbi.kun.nl/pub/molbio/data/dssp/1est.dssp

 Information on the PDB HELIX/SHEET format was obtained from:
 http://www.wwpdb.org/documentation/format32/sect5.html

"""

import sys

# author: Andrew Jewett
g_program_name = __file__.split('/')[-1]
g_date_str = '2012-12-10'
g_version_str = '0.8.0'



def Int2Digits(i, base):
    """
    The next function converts the number into a list of digits
    For example:
    if base=10, then Int2Digits(126, 10) = [1, 2, 6]
    if base=2,  then Int2Digits(126, 2)  = [1, 1, 1, 1, 1, 1, 0]
    (This function is called by the Number2String() function.)
    """

    if (i < base):
        return [i]
    else:
        return Int2Digits(i//base, base) + [i % base]



# The next function converts 1-letter amino acid names into 3-letter names.
# If the user doesn not specify a PDB file (with 3-letter res names), use this.
def ResNamesFrom1Char(c):
    lookup = {'A':'ALA',
              'R':'ARG',
              'N':'ASN',
              'D':'ASP',
              'C':'CYS',
              'E':'GLU',
              'Q':'GLN',
              'G':'GLY',
              'H':'HIS',
              'I':'ILE',
              'L':'LEU',
              'K':'LYS',
              'M':'MET',
              'F':'PHE',
              'P':'PRO',
              'S':'SER',
              'T':'THR',
              'W':'TRP',
              'Y':'TYR',
              'V':'VAL'}
    if (lookup.has_key(c)):
        return lookup[c]
    else:
        return c+c+c



# For some reason, I also have to give each helix/strand a 3-character name.
def Number2String(i):
    str = ''
    for d in Int2Digits(i, 26):
        c = chr(ord('A') + d)
        str += c
    return str




def IsHelix(secondary_type):
    return (secondary_type in 'HIG')

def IsStrand(secondary_type):
    return (secondary_type in 'E')

def SameType(secondary_type1, secondary_type2):
    return (secondary_type1 == secondary_type2)

#def SameType(secondary_type1, secondary_type2):
#    return ((IsHelix(secondary_type1) and IsHelix(secondary_type2))
#            or
#            (IsStrand(secondary_type1) and IsStrand(secondary_type2)))


def PrintPDB(secondary_type,
             start_chainID,
             start_resSeq,
             start_iCode,
             start_resName,
             stop_chainID,
             stop_resSeq,
             stop_iCode,
             stop_resName,
             average_chirality,
             length,
             helix_counter,
             sheet_counter):

    if (IsHelix(secondary_type)):
        comment = ' generated by DSSP & dssp2pdb '
        class_code = ' 1' #PDB code corresponding to a right-handed alpha helix
        if (average_chirality < 0.0):
            class_code = ' 7' # left handed alpha helix
        if (secondary_type == 'I'):
            class_code = ' 3' # right-handed pi helix
        elif (secondary_type == 'G'):
            class_code = ' 5' # right-handed 3-10 helix
        sys.stdout.write("HELIX  "+
                         str(helix_counter).rjust(3)+" "+
                         str(helix_counter).rjust(3)+" "+
                         #Number2String(helix_counter-1).rjust(3)+" "+
                         start_resName+" "+start_chainID+" "+
                         str(start_resSeq).rjust(4)+start_iCode+" "+
                         stop_resName+" "+stop_chainID+" "+
                         str(stop_resSeq).rjust(4)+stop_iCode+
                         class_code + " " + 
                         comment + str(length).rjust(5)+"\n")

    if (IsStrand(secondary_type)):
        num_strands_in_sheet = 1
        sense = ' 0'
        sys.stdout.write("SHEET  "+
                         str(sheet_counter).rjust(3)+" "+
                         #str(sheet_counter).rjust(3)+" "+
                         Number2String(sheet_counter-1).rjust(3)+
                         str(num_strands_in_sheet).rjust(2)+" "+
                         start_resName+" "+start_chainID+
                         str(start_resSeq).rjust(4)+start_iCode+" "+
                         stop_resName+" "+stop_chainID+
                         str(stop_resSeq).rjust(4)+stop_iCode+
                         sense+
                         "                                   "+"\n")
    



def main():

    min_length_strand = 0
    min_length_helix  = 0

    resNamesFromPDBid = {}

    if (len(sys.argv) == 2):
        pdb_file_name = sys.argv[1]
        # Then open the PDB file and load the names of each residue 
        for line in open(pdb_file_name,'r'):
            if (line[0:6] == 'ATOM  '):
                # the next 3 data identify the residue to which the atom belongs
                chainID = line[21]
                resSeq  = int(line[22:26])
                iCode   = line[26]
                # this next short string specifies the type of this residue
                resName = line[17:20]

                resNamesFromPDBid[(chainID, resSeq, iCode)] = resName


    elif (len(sys.argv) != 1):
        sys.stderr("Error: wrong number of arguments.\n")
        exit(-1)



    # Now, read the DSSP file:

    dssp_lines = sys.stdin.readlines()
    prev_secondary_type = ' '



    # First, skip over the description at the begining of the DSSP file
    skip = 0
    for i in range(0,len(dssp_lines)):
        line = dssp_lines[i]
        if (line[0:25] == '  #  RESIDUE AA STRUCTURE'):
            skip = i+1

    dssp_lines = dssp_lines[skip:]


    helix_counter  = 0
    sheet_counter  = 0


    # Finally, scan over the (remaining lines of the) DSSP file:

    for i in range(0,len(dssp_lines)):

        line = dssp_lines[i]
        #sys.stderr.write("line = \""+line.rstrip()+"\"\n")

        line_contains_junk = False
        line_contains_digits = False
        for c in line[6:10]:
            if (c in '0123456789'):
                line_contains_digits = True
            elif (c != ' '):
                line_contains_junk = True
        if (not line_contains_digits):
            line_contains_junk = True


        if (not line_contains_junk):

            # Lookup the 3-part identifier for this residue (used in PDB files)
            chainID = line[11]       #To which chain does this residue belong?
            resSeq  = int(line[6:10])#This integer gives a rough idea of the 
                                     #location of a residue in the chain's sequence.
            iCode   = line[10]       #A 1-character code for inserting residues

            secondary_type = line[16]
            chirality_code = line[22]

            # What is the name of this residue? (This should be a 3-character string)
            if (len(resNamesFromPDBid) > 0):
                resName = resNamesFromPDBid[(chainID,resSeq,iCode)]#lookup from PDB file
            else:
                resName = ResNamesFrom1Char(line[13]) #convert 1-lttr to 3-lttr code

            # Check for my stupidity
            if (len(resName) != 3):
                sys.stderr.write("Internal Error: Illegal residue name: \""+name+"\"\n"\
                                     "  for residue (chain\""+chainID+"\", res="+str(resSeq)+" iCode=\""+iCode+"\")\n"\
                                     "  Please report this bug (along with input files).\n")
                exit(-1)



            # Does this new residue mark the end of a helix or a strand?

            # If the previous residue belonged to a helix or strand...
            if (IsHelix(prev_secondary_type) or
                IsStrand(prev_secondary_type)):

                # ...and if the current residue does -not- also share the same
                # kind of secondary structure (either helix or strand),
                # ...(or if it belongs to a different chain)
                # ...THEN this new residue marks the end of that helix/strand
                if ((not SameType(secondary_type, 
                                  prev_secondary_type)) or
                    (chainID != prev_chainID)):

                    # ... Check for one more thing:
                    #     if the helix/sheet is too short, then ignore it
                    if (IsHelix(prev_secondary_type) and
                        (length < min_length_helix)):
                        helix_counter -= 1 #-1 because we should not have counted it
                    elif (IsStrand(prev_secondary_type) and
                        (length < min_length_strand)):
                        sheet_counter -= 1 #-1 because we should not have counted it
                    else:
                        # If there are no problems, print out that helix/strand
                        PrintPDB(prev_secondary_type,
                                 start_chainID,
                                 start_resSeq,
                                 start_iCode,
                                 start_resName,
                                 prev_chainID,
                                 prev_resSeq,
                                 prev_iCode,
                                 prev_resName,
                                 total_chirality / length,
                                 length,
                                 helix_counter,
                                 sheet_counter)
                else:
                    # Keep track of the length of each helix/strand
                    length += 1
                    # Was it right handed '+' or left handed '-' ?
                    if (chirality_code != '-'):
                        total_chirality += 1.0
                    else:
                        total_chirality -= 1.0




            # Is this the beginning of a new helix/strand?
            if ((IsHelix(secondary_type) or 
                 IsStrand(secondary_type)) and
                (not SameType(secondary_type, prev_secondary_type))):

                start_resSeq  = resSeq
                start_iCode   = iCode
                start_chainID = chainID
                start_resName = resName
                length = 1
                if (IsHelix(secondary_type)):
                    helix_counter += 1

                if (IsStrand(secondary_type)):
                    sheet_counter += 1

                total_chirality = 0.0


            # keep track of the properties of this residue
            prev_secondary_type = secondary_type
            prev_resSeq    = resSeq
            prev_iCode     = iCode
            prev_chainID   = chainID
            prev_resName   = resName




    # If we reached the end of the file / molecule, then we have obviously
    # reached the end of whatever helix or strand which was located at the end of
    # that molecule.  We should get around to printing out that helix/strand now:

    if (IsHelix(secondary_type) or IsStrand(secondary_type)):
        PrintPDB(prev_secondary_type,
                 start_resSeq,
                 start_iCode,
                 start_chainID,
                 start_resName,
                 prev_resSeq,
                 prev_iCode,
                 prev_chainID,
                 prev_resName,
                 total_chirality / length,
                 length,
                 helix_counter,
                 sheet_counter)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This removes lines from pdb files which define 
where HELIX, SHEET, or TURNs are located.
"""

import sys

def main():
    for line in sys.stdin:
        if ((line[0:6] != "HELIX ") and 
            (line[0:6] != "SHEET ") and
            (line[0:5] != "TURN ")):
            sys.stdout.write(line)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
 Typical usage (example):

    dlpisces.py < cullpdb_pc40_res2.5_R1.0_d090206_chains9151

 This program processes lists of "culled pdb" files generated by the 
 PISCES server at 
   http://dunbrack.fccc.edu/PISCES.php
   http://dunbrack.fccc.edu/Guoli/pisces_download.php
 This server uses a method described in:
 G. Wang and R. L. Dunbrack, Jr. 
 "PISCES: a protein sequence culling server."
  Bioinformatics, 19:1589-1591, 2003

 These lists contain non-redundant subsets of all the PDB files
 in the rcsb repository (obtained by crystallography?) whose
 resolution (in Angstroms) and R-values lie below some 
 desired threshold.  Different lists correspond to
 different levels of tolerable sequence homology, as well
 as different tolerated cutoffs for resolution and R-values.

 Here is an except from one of these files:
"cullpdb_pc40_res2.5_R1.0_d090206_chains9151"
 -----------------------
IDs         length Exptl.  resolution  R-factor FreeRvalue
7ODCA       424  XRAY        1.600    0.20    0.23
2AXPA       173  XRAY        2.500    0.26    0.29
2VKEA       207  XRAY        1.620    0.25    0.30
1IHJA        98  XRAY        1.800    0.22    0.24
2NUTC       196  XRAY        2.300    0.22    0.27
1DQTA       117  XRAY        2.000    0.23    0.25

 Every line in this file (other than the comment line at the beginning)
 begins with a 4-letter pdb identifier, immediately followed by a 1-letter
 chain identifier.  Thus it's possible for the same 4-letter PDB indentifier
 to appear twice in this list (with different chains).
 Each 4-letter pdb code is compared with a list of pdb codes downloaded so far
(which is initially read from the file "pdbs_most_recent.txt", if present).
If it is new 4-letter pdb code, then the corresponding pdb file is downloaded,
and appended to the end of the "pdbs_most_recent.txt" file.
The corresponding DSSP file is also downloaded, if available.

Finally, the relevant chain is extracted from each PDB file and placed
in a new PDB file like 7odc_chainA.pdb

"""

# author: Andrew Jewett
g_program_name = __file__.split('/')[-1]
g_date_str = '2017-4-21'
g_version_str = '0.5.1'


import sys, urllib2, random, time, gzip


def FileExists(fname):
    try:
        file = open(fname, 'r')
    except IOError:
        exists = False
    else:
        exists = True
    return exists



def ExtractFileName(url):
    # The name of the file we create will be the same as the last few 
    # characters of the URL. (Whatever comes after the last '/' character.)
    out_fname = url
    slash_loc = out_fname.rfind('/')
    if (slash_loc >= 0):  # if '/' was found, then remove text preceeding '/'
        out_fname = out_fname[slash_loc+1:]
        if (out_fname == ''):
            sys.stderr.write('Error: filename difficult to determine from URL:\n' +\
                                 '       \"'+url+'\"\n')
            sys.exit(-1)
    return out_fname



# This function downloads a file from a URL, and (if requested by the user)
# prints out a warning message if there was a problem.

def DownloadFileTo(url, file_name, verbose_mode = True):
    # Open the input file (at url) for reading
    # Simple way:
    #   in_file = urllib.urlopen(url)
    # But the web site may return garbage if the file doesn't exist.
    # Instead, we check for errors first:
    request = urllib2.Request(url)
    try: in_file = urllib2.urlopen(request)
    except urllib2.URLError, e:
        if (verbose_mode):
            sys.stderr.write(str(e)+'\n')
            sys.stderr.write('   omitting file \"'+file_name+'\"\n')
    else:
        if (verbose_mode):
            sys.stderr.write('downloading file \"'+file_name+'\"\n')
        # Open the output file for writing
        out_file = open(file_name, 'wb')
        out_file.write(in_file.read())
        out_file.close()



# This version infers the file name from the URL.
def DownloadFile(url, verbose_mode = True):
    DownloadFileTo(url, ExtractFileName(url), verbose_mode)




def main():
    # Read in the list of PDB files already downloaded:

    pdbs_old = set([])
    try:
        pdbs_old_file = open('pdbs_old.txt', 'r')
        for pdb_code in pdbs_old_file:
            pdb_code = pdb_code.strip()#eliminate trailing and preceeding whitespace
            pdb_code = pdb_code.lower()#pdb codes are case-insensitive
            if (len(pdb_code) != 4):
                sys.stderr.write('Error in in \"pdbs_old.txt\":\n')
                sys.stderr.write('      Invalid PDB-code: \"'+pdb_code+'\"\n')
                exit(-1)
            pdbs_old.add(pdb_code)
        pdbs_old_file.close()
    except IOError:
        pass




    # Here we keep track of the list of pdb files which
    #  i) are in the current list of pdb files requested in 
    #     sys.stdin
    # ii) are in the current list, but are not downloaded yet 
    #     (new pdb files)
    pdbs_current_file = open('pdbs_most_recent.txt', 'wb')
    pdbs_current = set([]) #entire list of pdb codes requested
    pdbs_new  = set([]) #a list of new pdb codes requested that were not in the old list
    pdbs_old_file = open('pdbs_old.txt', 'a')


    pisces_list = sys.stdin.readlines()
    for line in pisces_list:

        #Check to make sure the line does not contain comments.
        #Comments include the string "R-factor".

        if (line.find('R-factor') == -1):
            pdb_code = line[0:4]
            if (len(pdb_code) != 4):
                sys.stderr.write('Error:  Invalid PDB-code: \"'+pdb_code+'\"\n')
                exit(-1)

            pdb_code = pdb_code.lower()  #(pdb codes are case-insensitive)

            if (pdb_code in pdbs_new):
                sys.stderr.write(pdb_code+' appears redundantly. skipping\n')
            elif (pdb_code in pdbs_old):
                if (not (pdb_code in pdbs_current)):
                    sys.stderr.write(pdb_code+' downloaded already. skipping.\n')
                    pdbs_current.add(pdb_code)
                    pdbs_current_file.write(pdb_code+'\n')
                    pdbs_current_file.flush()  #<- necessary in case we get interrupted
                else:
                    sys.stderr.write(pdb_code+' appears redundantly. skipping\n')

            else:
                #Download the corresponding pdb file
                file_name = pdb_code+'.pdb.gz' # <- these are compressed files
                url = 'http://www.rcsb.org/pdb/files/'+file_name
                # Note, if the URL above files, try this one instead:
                # url = 'https://files.rcsb.org/download/'+file_name
                DownloadFileTo(url, file_name)

                #Unzip the pdb file
                with gzip.open(file_name, 'rb') as f:
                    file_content = f.read(f)
                    f.close()
                f = open(pdb_code+'.pdb', 'wb'):
                f.write(file_content)
                if not FileExists(pdb_code+'.pdb'):
                    sys.stderr.write('Error: A problem occured when trying to download PDB code \"'+pdb_code+'\"\n'
                                     '       Delete this entry from the file \"pdbs_old.txt\", and rerun '+g_program_name+'\n')
                    sys.exit(-1)

                #Optional: Download the corresponding DSSP file
                url = 'ftp://ftp.cmbi.ru.nl/pub/molbio/data/dssp/'+pdb_code+'.dssp'
                DownloadFile(url)
                if not FileExists(pdb_code+'.dssp'):
                    sys.stderr.write("    (The old DSSP PDB is server flaking out again.\n"
                                     "     Don't worry.  DSSP files are not needed.)\n")

                #Keep track of the the pdbs we have downloaded so far:

                pdbs_current.add(pdb_code)
                pdbs_current_file.write(pdb_code+'\n')
                pdbs_current_file.flush()  #<- necessary in case we get interrupted
                pdbs_old_file.write(pdb_code+'\n')
                pdbs_old_file.flush()      #<- necessary in case we get interrupted
                pdbs_new.add(pdb_code)


    pdbs_current_file.close()

    def ChainFromPDBfile(chainID, pdb_file):
        for line in pdb_file:
            line_type = line[0:6]
            if line_type in set(['ATOM  ', 'HETATM', 'ANISOU', 'SIGATM', 'SIGUIJ']):
                if line[21:22] == chainID:
                    yield line
            elif line_type == "HET   ":
                if line[12:13] == chainID:
                    yield line
            elif line_type == "HELIX ":
                initChainID = line[19:20]
                endChainID  = line[31:32]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "SHEET ":
                initChainID = line[21:22]
                endChainID  = line[32:33]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "TURN  ":
                initChainID = line[19:20]
                endChainID  = line[30:31]
                if (initChainID == chainID) or (endChainID == chainID):
                    yield line
            elif line_type == "SEQRES":
                if line[11:12] == chainID:
                    yield line
            elif line_type == "TER   ":
                ter_chainID = line[21:22]
                if (ter_chainID == chainID) or (ter_chainID == " "):
                    yield line
            else:
                yield line
        return


    # Now loop through the list of chains in the pisces_list again
    # and exctract each chain from it's corresponding PDB file.

    pdb_files_file = open('pdb_files.txt','wb')

    for line in pisces_list: #<-reading the html file generated by www.rcsb.edu

        #Check to make sure the line does not contain comments.
        #Comments include the string "R-factor".
        if (line.find('R-factor') == -1):
            pdb_code = line[0:4]
            pdb_code = pdb_code.lower()  #(pdb codes are case-insensitive)
            assert(len(pdb_code) == 4)
            chainID  = line[4:5]
            try:
                pdb_file = open(pdb_code+'.pdb', 'r')
            except IOError:
                sys.stderr.write('Error: A problem occured when trying to download PDB code \"'+pdb_code+'\"\n'
                                 '       Delete this entry from the file \"pdbs_old.txt\", and rerun dlpisces.py\n')
                sys.exit(-1)
            # If not already present, then create a new PDB 
            # file containing only the requested chain.
            new_filename = pdb_code+'_chain'+chainID+'.pdb'
            try:
                pdb_file_chain = open(new_filename, 'r')
                pdb_file_chain.close()
            except IOError:
                sys.stderr.write('creating file \"'+new_filename+'\"\n')
                pdb_file_chain = open(new_filename, 'w')
                for line in ChainFromPDBfile(chainID, pdb_file):
                    pdb_file_chain.write(line)
                pdb_file_chain.close()
            pdb_file.close()
            pdb_files_file.write(new_filename+'\n')
    pdb_files_file.close()

    # Here, we help the user keep track of the pdb files which 
    # were in the original list, but are not in the current list:

    pdbs_not_needed_file = open('pdbs_not_needed.txt', 'wb')
    for pdb_code in pdbs_old:
        if (not (pdb_code in pdbs_current)):
            pdbs_not_needed_file.write(pdb_code+'\n')
    pdbs_not_needed_file.close()



if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
 This is a crude script which prints out the coordinates for the average 
 position of each residue in a PDB file (three numbers per line).
 (Note: This program does not calculate the center-of-mass.  All atoms 
        are given the same weight when calculating the average position.)

 Typical usage: 

 pdb2coords_ave.py < PDB_FILE

 Exceptions:
   Backbone atoms (other than alpha-carbon " CA " atoms) are ignored.
   Glycine residues are also ignored.

 These settings can not be changed without editing the code for this script.
 (unlike other scripts whose behavior can be changed by command-line arguments)

 Output:
   The number of lines of output should match the number of residues.
   For glycines (and residues lacking elligible atoms) a blank line is printed.
   For all other residues, the x, y, and z coordinates of the average
   position are printed (3 numbers), one line per residue.

"""

import sys
from operator import attrgetter
from resid import *


# --- THE FOLLOWING FEATURES (interval restrictions) may be removed later:--
#
# The entire PDB file does not have to be used.  Instead, the user can pass 
# 6 arguments denoting the first and last residue of an interval in the chain.
# In order to specify any amino acid in a PDB file, you must provide 3
# identifiers:
#   the ChainID a single letter specifying
#   the SeqNum an integer indicating the location within that chain
#   the ICode (an insert code, usually " ")
#
# All 3 identifiers are needed for both the starting and ending residues.
# Consequently this program expects 6 arguments: 
#    (to be read from the command line)
# ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last


# The following atoms are not included in the average:
ignore_these_atoms    = [' N  ', ' H  ', ' C  ', ' O  ', ' CA ']
# The following residues are completely excluded:
ignore_these_residues = set(['GLY', 'PRO'])



def main():
    use_all_residues = True

    if len(sys.argv) > 1:
        i = 1
        while i < len(sys.argv):

            if (len(sys.argv[i]) == 3):
                # Add the string to the list of amino acids we want to ignore
                resType = sys.argv[i]
                ignore_these_residues.add(resType)
                i += 1

            else: # if not is_digit(sys.argv[i][0])

                # If the next argument is a number, then interpret this number
                # as a residue sequence number
                if len(sys.argv) < i+6:
                    sys.stderr.write("Error: Not enough arguments or argument type error:\n"
                                     "       Offending arguemt #"+str(i)+": \""+sys.argv[i]+"\"\n")

                    if len(sys.argv[i]) == 1:
                        sys.stderr.write("\n"
                                         "       Is argument \""+sys.argv[i]+"\" a chain ID letter?\n"
                                         "       (Chain IDs are only passed as arguments when you want to limit the\n"
                                         "        residues considered to within an interval in the PDB file.\n"
                                         "        To specify an interval, you must provide 5 more aruments.  See below.)\n"
                                         "\n"
                                         "       Note: one-letter residue codes (eg A,V,L,...) are not considered\n"
                                         "             valid residue types by this program.\n"
                                         "             You must specify a 3-letter equivalent (eg ALA, VAL, LYS,...).\n"
                                         "             One-letter arguments are interpreted as chain-IDs. (See below.)\n"
                                         "       \n"
                                         "       By default, the the sequence is extracted from the entire PDB file.\n"
                                         "       In that case, no arguments are required.\n"
                                         "       Alternately, you can limit the selection to a single interval of\n"
                                         "       residues from one of the chains in the PDB file.\n"
                                         "       To specify an interval, you must passing 6 arguments to this program.\n"
                                         "       This program requires a pair of residues to designate the first and\n"
                                         "       last members of the interval.  Each residue requires 3 identifiers.\n"
                                         "       Consequently the six arguments needed are:\n"
                                         "\n"
                                         "ChainID_first SeqNum_first ICode_first ChainID_last SeqNum_last ICode_last\n"
                                         "\n")
                    else:
                        sys.stderr.write("\n"
                                         "Note: PDB files are not passed as arguments\n"
                                         "      but are read from the standard input, for example using the notation\n"
                                         "\n"
                                         "      "+sys.argv[0]+" < file.pdb\n")
                    exit(-1)
                else:
                    use_all_residues = False
                    first = ResID(sys.argv[i], int(sys.argv[i+1]), sys.argv[i+2])
                    last  = ResID(sys.argv[i+3], int(sys.argv[i+4]), sys.argv[i+5])
                    #sys.stderr.write('  Interval selected: (\"'+first.chainID+'\", '+str(first.seqNum)+', \"'+first.iCode+'\") ... (\"'+last.chainID+'\", '+str(last.seqNum)+', \"'+last.iCode+'\")\n')
                    i += 6

    resID2pos = {}

    for line in sys.stdin:
        if (line[0:6] == "ATOM  "):
            #atomID    = int(line[6:11])
            atomType  = line[12:16]
            #altLoc    = line[16:17]
            resType   = line[17:20]
            chainID   = line[21:22]
            resSeq    = line[22:26]
            iCode     = line[26:27]
            resID = ResID(chainID, int(resSeq), iCode)
            #xyz_str   = line[30:54]<-not safe. spaces not always in between numbers
            x_str     = line[30:38].strip()
            y_str     = line[38:46].strip()
            z_str     = line[46:54].strip()
            x = float(x_str)
            y = float(y_str)
            z = float(z_str)
            if (use_all_residues or ((first<=resID) and (resID<=last))):
                if resID not in resID2pos:
                    resID2pos[resID] = []

                # Ignore atoms on the backbone (other than CA), and also 
                # ignore all atoms which are not heavy atoms (hydrogen atoms).
                if ((resType not in ignore_these_residues) and
                    (atomType not in ignore_these_atoms)):
                    resID2pos[resID].append([x, y, z, atomType])

    # Extract an (unordered) list of the resIDs of the residues in the sequence
    resIDs = [resID for resID in resID2pos]

    # Residues in PDB files are often not listed in order.
    # Consequently, we must sort the list by chainID, seqNum, and finnaly iCode:
    sequence_of_resIDs = sorted(resIDs, key=attrgetter('chainID','seqNum','iCode'))

    # Now loop through the sequence of resIDs, and calculate 
    # the average position of the atoms in that residue.

    for resID in sequence_of_resIDs:
        coords = resID2pos[resID]
        if coords == []:
            sys.stdout.write('\n')
        else:
            xyz_tot = [0.0 for d in range(0,3)]
            num_atoms_in_res = len(coords)
            for i in range(0, num_atoms_in_res):

                for d in range(0,3):
                    xyz_tot[d] += coords[i][d]

            x_ave = xyz_tot[0] / num_atoms_in_res
            y_ave = xyz_tot[1] / num_atoms_in_res
            z_ave = xyz_tot[2] / num_atoms_in_res


            x_str = str(x_ave)
            y_str = str(y_ave)
            z_str = str(z_ave)

            # Alternately, I could have used:
            # x_str = "%5.3" % x_ave
            # y_str = "%5.3" % y_ave
            # z_str = "%5.3" % z_ave

            sys.stdout.write(x_str+' '+y_str+' '+z_str+'\n')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program finds each chain in a PDB file containing DNA residues (with 
backbone atoms: " C1'",...," C5'"," O3'"," O4'"," O5'"," P  "," OP1"," OP2")
... and create a new file containing only that chain.
PDB-files lacking DNA residues will be ignored.
The files which are created will match the name of the original file, with
the .pdb or .PDB extension stripped off (if present), and replaced with 
_?.pdb,  Where "?" is the chain letter.
"""

import sys

g_program_name  = __file__.split('/')[-1]
g_date_str     = '2015-8-17'
g_version_str  = '0.5.0'


usage_string = "Usage:\n\n  "+g_program_name+" file1.pdb [file2.pdb file3.pdb ...]\n\n"

usage_string += \
"""
Explanation:

"""
usage_string += __doc__


res_types = ([' DA',
              ' DG',
              ' DC',
              ' DT'])


heavy_atoms = (' C1\'',
               ' C2\'',
               ' C3\'',
               ' C4\'',
               ' C5\'',
               ' O3\'',
               ' O4\'',
               ' O5\'',
               ' P  ',
               ' OP1',
               ' OP2')


def ExtractChainText(file_name):
    """
    ExtractChainText() finds all of the chains in a PDB file which contain DNA
    nucleotides and creates a new PDB file for that chain with a similar name.

    """

    pdb_file = open(file_name, 'r')

    chain_atoms_found = {}
    chain_text = {}

    for line in pdb_file:
        if line[0:6] == 'ATOM  ':
            chainID = line[21:22]
            if (not chainID in chain_atoms_found):
                chain_atoms_found[chainID] = {}
                chain_text[chainID] = []
                for atom_name in heavy_atoms:
                    chain_atoms_found[chainID][atom_name] = False

            chain_text[chainID].append(line)

            atom_type = line[12:16]
            res_type = line[17:20]
            if res_type in res_types:
                #if ((atom_type in atoms_found) and (not atoms_found[atom_type])):
                #    print('found atom_type=\"'+atom_type+'\", in res_type=\"'+res_type+'\"')
                chain_atoms_found[chainID][atom_type] = True


    for chainID in chain_atoms_found:
        search_criteria_satisfied = True
        for atom_type in chain_atoms_found[chainID]:
            if (not chain_atoms_found[chainID][atom_type]):
                search_criteria_satisfied = False
        if search_criteria_satisfied:
            sys.stderr.write("  Chain \""+chainID+"\" contains DNA.\n")
            # Then create a new PDB file with a name similar to the original:
            pdb_file_chain_name = file_name
            i = pdb_file_chain_name.lower().rfind('.pdb')
            if i != -1:
                pdb_file_chain_name = (pdb_file_chain_name[:i] +
                                       '_' + chainID +
                                       pdb_file_chain_name[i:])
            else:
                pdb_file_chain_name = file_name + '_' + chainID
            sys.stderr.write('    Creating file \"'+pdb_file_chain_name+'\"\n')
            pdb_file_chain = open(pdb_file_chain_name, 'w')
            pdb_file_chain.write(''.join(chain_text[chainID]))
            pdb_file_chain.close()

    pdb_file.close()



def main():
    if len(sys.argv) < 2:
        sys.stderr.write("Error: Expected at least one argument.\n")
        sys.stderr.write(usage_string)
        exit(1)

    sys.stderr.write(g_program_name+' v'+g_version_str+' '+g_date_str+'\n')

    for file_name in sys.argv[1:]:
        sys.stderr.write('  Reading file \"'+file_name+'\"\n')
        ExtractChainText(file_name)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This returns whether or not a PDB file contains all of the required backbone
heavy atoms: " C1'",...," C5'"," O3'"," O4'"," O5'"," P  "," OP1"," OP2"
"""

import sys

res_types = ([' DA',
              ' DG',
              ' DC',
              ' DT'])


atoms_found = {' C1\'':False,
               ' C2\'':False,
               ' C3\'':False,
               ' C4\'':False,
               ' C5\'':False,
               ' O3\'':False,
               ' O4\'':False,
               ' O5\'':False,
               ' P  ':False,
               ' OP1':False,
               ' OP2':False}

def main():
    for line in sys.stdin:
        if (line[0:5] == 'ATOM '):
            atom_type=line[12:16]
            res_type=line[17:20]
            if (res_type in res_types):
                #if ((atom_type in atoms_found) and (not atoms_found[atom_type])):
                #    print('found atom_type=\"'+atom_type+'\", in res_type=\"'+res_type+'\"')
                atoms_found[atom_type] = True

    search_criteria_satisfied = True
    for atom_type in atoms_found:
        if (not atoms_found[atom_type]):
            search_criteria_satisfied = False

    if (search_criteria_satisfied):
        exit(0)  # normal termination indicates all atoms were found
    else:
        exit(1)  # non-zero (abnormal termination) exit code indicates this PDB
                 # file does not contain very many heavy atoms in amino acids

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This program currently does not do anything which grep could not do.
This returns prints nothing, but it returns (using an exit code)
whether or not a PDB file contains a "TURN" record.
If it doesn't it could either be because the original authors of the 
PDB file neglected to indicate where the turns are, or because
there simply arne't any.

This program was not meant to be invoked explicitly by the user.
(In the future, perhaps I will eliminate it.)
"""

import sys

def main():
    turn_found = False;
    for line in sys.stdin:
        if (line[0:5] == "TURN "):
            turn_found = True;

    if (turn_found):
        exit(0)  # normal termination indicates a turn was found
    else:
        exit(1)  # non-zero (abnormal termination) exit code indicates 
                 # this PDB file does not contain a turn.

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This returns whether or not a PDB file contains all of the require
backbone heavy atoms: ' CA ', ' CB ', ' C  ', ' N  ', ' O  '
"""

import sys

res_types = (['ALA',
              'ARG',
              'ASN',
              'ASP',
              'CYS',
              'GLU',
              'GLN',
              'GLY',
              'HIS',
              'ILE',
              'LEU',
              'LYS',
              'MET',
              'PHE',
              'PRO',
              'SER',
              'THR',
              'TRP',
              'TYR',
              'VAL'])


atoms_found = {' CA ':False,
               ' CB ':False,
               ' C  ':False,
               ' N  ':False,
               ' O  ':False}

def main():
    for line in sys.stdin:
        if (line[0:5] == 'ATOM '):
            atom_type=line[12:16]
            res_type=line[17:20]
            #print('atom_type=\"'+atom_type+'\", res_type=\"'+res_type+'\"')
            if (res_type in res_types):
                atoms_found[atom_type] = True

    search_criteria_satisfied = True
    for atom_type in atoms_found:
        if (not atoms_found[atom_type]):
            search_criteria_satisfied = False

    if (search_criteria_satisfied):
        exit(0)  # normal termination indicates all atoms were found
    else:
        exit(1)  # non-zero (abnormal termination) exit code indicates this PDB
                 # file does not contain very many heavy atoms in amino acids

if __name__ == "__main__":
    main()



from setuptools import setup

setup(
  
  name='dlpdb',

  packages=['dlpdb'],

  author='Andrew Jewett',

  author_email='jewett.aij@gmail.com',

  description='collect statistics from the entire PDB library',

  long_description='A collection of tiny scripts to help automate the process of downloading and extracting coordinates, sequences, and secondary-structure information from (a representative subset of) structures from the entire PDB library.  These scripts were originally used to estimate distances distances and angles between specific atoms in proteins and DNA.  The distribution of these distances and angles can be used to estimate force-field parameters for coarse grained models for these biomolecules.',

  url='https://github.com/jewettaij/dlpdb',

  download_url='https://github.com/jewettaij/dlpdb/archive/v0.2.1.zip',

  version='0.2.1',

  # BSD 3-Clause License:
  # - http://choosealicense.com/licenses/bsd-3-clause
  # - http://opensource.org/licenses/BSD-3-Clause

  license='BSD',

  classifiers=['Development Status :: 4 - Beta',
               'License :: OSI Approved :: BSD License',
               'Environment :: Console',
               'Operating System :: MacOS :: MacOS X',
               'Operating System :: POSIX :: Linux',
               'Operating System :: Microsoft :: Windows'
  ],

  scripts=['dlpdb/scripts/extract_angles.sh',
           'dlpdb/scripts/extract_dihedrals.sh',
           'dlpdb/scripts/extract_distances.sh',
           'dlpdb/scripts/extract_helix_angles.sh',
           'dlpdb/scripts/extract_helix_dihedrals.sh',
           'dlpdb/scripts/extract_helix_distances.sh',
           'dlpdb/scripts/extract_helix_resAveDistances.sh',
           'dlpdb/scripts/extract_resAveDistances.sh',
           'dlpdb/scripts/extract_sheet_angles.sh',
           'dlpdb/scripts/extract_sheet_dihedrals.sh',
           'dlpdb/scripts/extract_sheet_distances.sh',
           'dlpdb/scripts/extract_sheet_resAveDistances.sh',
           'dlpdb/scripts/extract_turn_angles.sh',
           'dlpdb/scripts/extract_turn_dihedrals.sh',
           'dlpdb/scripts/extract_turn_distances.sh',
           'dlpdb/scripts/extract_turn_resAveDistances.sh',
           'dlpdb/scripts/move_membrane_proteins.sh',
           'dlpdb/scripts/move_missing_dna_heavy_atoms.sh',
           'dlpdb/scripts/move_missing_protein_heavy_atoms.sh',
           'dlpdb/scripts/move_missing_secondary_str.sh',
           'dlpdb/scripts/move_nmr_structures.sh',
           'dlpdb/scripts/move_non-dna.sh',
           'dlpdb/scripts/replace_all_secondary_str.sh',
           'dlpdb/scripts/replace_missing_secondary_str.sh'],

  entry_points={
      'console_scripts': ['coords2angles.py=dlpdb.coords2angles:main',
                          'coords2dihedrals.py=dlpdb.coords2dihedrals:main',
                          'coords2distances.py=dlpdb.coords2distances:main',
                          'coords2helixAngleOmega.py=dlpdb.coords2helixAngleOmega:main',
                          'dlpisces.py=dlpdb.dlpisces:main',
                          'download_pdbs.py=dlpdb.download_pdbs:main',
                          'dssp2pdb.py=dlpdb.dssp2pdb:main',
                          'fix_dna_residue_order_pdb.py=dlpdb.fix_dna_residue_order_pdb:main',
                          'has_dna_heavy_atoms.py=dlpdb.has_dna_heavy_atoms:main',
                          'has_helices.py=dlpdb.has_helices:main',
                          'has_protein_heavy_atoms.py=dlpdb.has_protein_heavy_atoms:main',
                          'has_rna_heavy_atoms.py=dlpdb.has_rna_heavy_atoms:main',
                          'has_secondary_str.py=dlpdb.has_secondary_str:main',
                          'has_sheets.py=dlpdb.has_sheets:main',
                          'has_turns.py=dlpdb.has_turns:main',
                          'helixAngleOmega.py=dlpdb.helixAngleOmega:main',
                          'merge_lines_periodic.py=dlpdb.merge_lines_periodic:main',
                          'pdb2coords_ave.py=dlpdb.pdb2coords_ave:main',
                          'pdb2coords.py=dlpdb.pdb2coords:main',
                          'pdb2helix.py=dlpdb.pdb2helix:main',
                          'pdb2sequence.py=dlpdb.pdb2sequence:main',
                          'pdb2sheet.py=dlpdb.pdb2sheet:main',
                          'pdb2turn.py=dlpdb.pdb2turn:main',
                          'pdb_interleave_residues.py=dlpdb.pdb_interleave_residues:main',
                          'select_chains_with_dna.py=dlpdb.select_chains_with_dna:main',
                          'select_interval.py=dlpdb.select_interval:main',
                          'strip_secondary_str.py=dlpdb.strip_secondary_str:main',
                          'truncate_chars.py=dlpdb.truncate_chars:main',
                          'truncate_tokens.py=dlpdb.truncate_tokens:main']},
  zip_safe=True,
  include_package_data=True
)





class ResID:
    def __init__(self, setChainID, setSeqNum, setICode):
        self.chainID = setChainID
        self.seqNum = setSeqNum
        self.iCode = setICode
    def __lt__(self, other):
        return ((self.chainID, self.seqNum, self.iCode) <
                (other.chainID, other.seqNum, other.iCode))
    def __le__(self, other):
        return not other < self
    def __gt__(self, other):
        return other < self
    def __ge__(self, other):
        return not self < other
    def __eq__(self, other):
        return (not self < other) and (not other < self)
    def __ne__(self, other):
        return (self < other) or (other < self)

    def __hash__(self):
        numChainIDs = 128
        numICodes = 128
        i = self.seqNum
        i *= numChainIDs
        i += ord(self.chainID)
        i *= numICodes
        i += ord(self.iCode)
        return i

    def __str__(self):
        return self.chainID+str(self.seqNum)+self.iCode

    def __repr__(self):
        return str(self)



#!/usr/bin/env python
"""
Explanation:

This program reads 2 pdb files (each of which should contain the same number
of residues).  It sorts them both according to the 3-part ResID identifier
(chainID, resSeq, iCode).  Then it prints out a new PDB file, alternately
printing 1 residue from 1 PDB file, with a residues from the other file,
(listed IN REVERSE ORDER).  The new chainID is set to 'X', and the resSeq
numbers will increase consecutively from 1 to 2N, where N is the number of
residues in either file.  This program was intended to be used to make it
easier to associate each nucleotide in double-stranded DNA with it's partner.

The user has to prepare the two PDB files to make sure that they contain two
different DNA strands which completely hybridize with each other (no overhangs),
AND the order must be reversed.  (Because the two strands in DNA are physically
oriented in opposite directions.)

"""

import sys
from operator import attrgetter
from collections import defaultdict
from resid import *


g_program_name = __file__.split('/')[-1]
g_date_str     = '2017-4-18'
g_version_str  = '0.9.0'


usage_string = "Usage:\n\n  "+g_program_name+" file1.pdb file2.pdb > file_merged.pdb\n\n"

usage_string += __doc__



def ResID2Text_from_pdb_file(file_name):

    """
    This function reads a pdb file, and creates a dictionary containing
    the text for each residue, indexed by it's identifier.  Identifiers
    in PDB files (ResIDs) have 3 parts (chainID,resSeq,iCode).

    """

    pdb_file = open(file_name, 'r')

    resID2text = defaultdict(list)
    
    chain_text = {}

    for line in pdb_file:
        #if line[0:6] in ("ATOM  ", "HETATM"):
        if line[0:6] in "ATOM  ":
            #atomID    = int(line[6:11])
            #atomType  = line[12:16]
            #altLoc    = line[16:17]
            #resType   = line[17:20]
            chainID   = line[21:22]
            resSeq    = line[22:26]
            iCode     = line[26:27]
            resID = ResID(chainID, int(resSeq), iCode)
            ##xyz_str=line[30:54] <- not safe. spaces not always between numbers
            #x_str     = line[30:38].strip()
            #y_str     = line[38:46].strip()
            #z_str     = line[46:54].strip()

            resID2text[resID].append(line)

    pdb_file.close()
    return resID2text




def SortedResidueText_from_pdb_file(file_name):

    """
    This function reads a pdb file, and creates a list of lists of strings.
    Each sub-list of strings contains the text for a particular residue.
    The residues are sorted according to their 3-part ResID identifiers
    (see above).

    """

    resID2text = ResID2Text_from_pdb_file(file_name)

    # Extract an (unordered) list of the resIDs of the residues in the sequence
    resIDs = [resID for resID in resID2text]

    # Residues in PDB files are often not listed in order.
    # Consequently, we must sort the list by chainID, seqNum, and finnaly iCode:
    resIDs_sorted = sorted(resIDs, 
                           key=attrgetter('chainID','seqNum','iCode'))

    index2text = []
    for resID in resIDs_sorted:
        index2text.append(resID2text[resID])

    return index2text


def main():
    if len(sys.argv) != 3:
        sys.stderr.write("Error: Expected two arguments (two pdb files)\n")
        sys.stderr.write(usage_string)
        exit(1)

    sys.stderr.write(g_program_name+' v'+g_version_str+' '+g_date_str+'\n')

    sys.stderr.write('  Reading file \"'+sys.argv[1]+'\"\n')
    strand1 = SortedResidueText_from_pdb_file(sys.argv[1])
    sys.stderr.write('  Reading file \"'+sys.argv[2]+'\"\n')
    strand2 = SortedResidueText_from_pdb_file(sys.argv[2])
    if (len(strand1) != len(strand2)):
        sys.stderr.write("Error: The two PDB files must have the same length.\n")
        exit(1)


    N = len(strand1)
    new_chainID=' '
    i = 1;
    for n in range(0, N):
        for line in strand1[n]:
            new_resSeq = str(i).rjust(4)
            new_line = (line[0:21] + new_chainID + new_resSeq + line[26:])
            # (leave the iCode in line[26:27] alone)
            sys.stdout.write(new_line)
        i += 1
        for line in strand2[N-n-1]:
            new_resSeq = str(i).rjust(4)
            new_line = (line[0:21] + new_chainID + new_resSeq + line[26:])
            # (leave the iCode in line[26:27] alone)
            sys.stdout.write(new_line)
        i += 1




if __name__ == "__main__":

    main()
